<?php
/**
 * 销售物料模型层
 *
 * @author kamisama.xia@gmail.com 
 * @version 0.1
 */

class material_mdl_sales_material extends dbeav_model{
    //是否有导出配置
    var $has_export_cnf = true;
    //导出的文件名
    var $export_name = '销售物料';

    var $defaultOrder = array('sm_id',' DESC');

    static public $__shopLists = array();

    /**
     * 销售物料类型字段格式化
     * @param string $row 物料类型字段
     * @return string
     */
    function modifier_sales_material_type($row){
        if($row == '1'){
            return '普通';
        }elseif($row == '2'){
            return '促销';
        }elseif($row == '3'){
            return '赠品';
        }elseif($row == '4'){
            return '福袋';
        }elseif($row == '5'){
            return '多选一';
        }else{
            return '-';
        }
    }

    /**
     * 物料是否可见字段格式化
     * @param string $row 物料是否可见字段
     * @return string
     */
    function modifier_shop_id($row){

        if(empty(self::$__shopLists)){
            self::$__shopLists['_ALL_'] = '全部店铺';
            $shopObj = app::get('ome')->model('shop');
            $tmp_shops = $shopObj->getList('shop_id,name',array(),0,-1);
            foreach($tmp_shops as $k => $shop){
                self::$__shopLists[$shop['shop_id']] = $shop['name'];
            }
        }
        
        return isset(self::$__shopLists[$row]) ? self::$__shopLists[$row] : $row;
    }

    //导出字段配置 移除不需要的字段
    public function disabled_export_cols(&$cols){
        unset($cols['column_edit']);
    }

    /**
     * 基础物料列表项扩展字段
     */
    function extra_cols(){
        return array(
            'column_retail_price' => array('label'=>'售价','width'=>'60','func_suffix'=>'retail_price'),
            'column_unit' => array('label'=>'包装单位','width'=>'60','func_suffix'=>'unit'),
            'colunm_associated_material' => array('label'=>'关联基础物料信息','width'=>'300','func_suffix'=>'associated_material'),
        );
    }

    /**
     * 售价扩展字段格式化
     */
    function extra_retail_price($rows){
        return kernel::single('material_extracolumn_salesmaterial_retailprice')->process($rows);
    }

    /**
     * 包装单位扩展字段格式化
     */
    function extra_unit($rows){
        return kernel::single('material_extracolumn_salesmaterial_unit')->process($rows);
    }

    /**
     * 关联的基础物料信息
     */
    function extra_associated_material($rows){
        return kernel::single('material_extracolumn_salesmaterial_associatedmaterial')->process($rows);
    }

    /**
    * 销售物料的过滤方法
    */
    public function _filter($filter,$tableAlias=null,$baseWhere=null){
        if (isset($filter['not_gift']) && $filter['not_gift'] == 1){
            $where .= '  AND sales_material_type IN (1,2)';
            unset($filter['not_gift']);
        }

        #对应店铺
        if(isset($filter['in_shop_id']) && !empty($filter['in_shop_id']))
        {
            if(!is_array($filter['in_shop_id']))
            {
                $filter['in_shop_id']    = explode(',', $filter['in_shop_id']);
            }
            foreach ($filter['in_shop_id'] as $key => $val)
            {
                if(empty($val)) unset($filter['in_shop_id'][$key]);
            }
            
            $where    .= " AND shop_id IN ('". implode("','", $filter['in_shop_id']) ."') ";
            unset($filter['in_shop_id']);
        }

        if($filter['slaes_material_type'] == 12){
            $where .= " AND sales_material_type IN (1,2)";
        }
        

        return parent::_filter($filter,$tableAlias,$baseWhere).$where;
    }

    /**
     * 导入模板的标题
     *
     * @param Null
     * @return Array
     */
    function exportTemplate($filter){
        foreach ($this->io_title($filter) as $v){
            $title[] = kernel::single('base_charset')->utf2local($v);
        }
        return $title;
    }

    /**
     * 导入导出的标题
     *
     * @param Null
     * @return Array
     */
    function io_title( $filter, $ioType='csv' ){
        switch( $filter )
        {
            case 'salesMaterial':
                $this->oSchema['csv'][$filter] = array(
                                                    '*:销售物料名称' => 'sales_material_name',
                                                    '*:销售物料编码' => 'sales_material_bn',
                                                    '*:物料类型' => 'sales_material_type',#普通、促销（暂时不支持）、 赠品 (只支持全部店铺)
                                                    '*:所属店铺' => 'shop_id',
                                                    
                                                    '*:物料成本价' => 'cost',
                                                    '*:物料售价' => 'retail_price',
                                                    '*:重量(g)' => 'weight',
                                                    '*:包装单位' => 'unit',
                                                    
                                                    '*:关联基础物料信息(促销类型多个以|竖线分隔)' => 'material_bn',
                                                    #普通的填写基础物料编号(促销类型格式：物料编号1:数量1|物料编号2:数量2)
                                                    #(福袋类型格式：组合名称：物料编码1|物料编码2-sku数量|送件数量|单品售价)
                                                    #(多选一类型格式：选择方式名称#物料编码1:排序值|物料编码2:排序值)
                                                );
                break;
        }
        
        $this->ioTitle[$ioType][$filter] = array_keys( $this->oSchema[$ioType][$filter] );
        return $this->ioTitle[$ioType][$filter];
    }

    /**
     * 准备导入的参数定义
     *
     * @param Null
     * @return Null
     */
    function prepared_import_csv(){
        $this->ioObj->cacheTime = time();
    }
    
    /**
     * 准备导入的数据主体内容部分检查和处理
     *
     * @param Array $data
     * @param Boolean $mark
     * @param String $tmpl
     * @param String $msg
     * @return Null
     */
    function prepared_import_csv_obj($data,$mark,$tmpl,&$msg = ''){
        return null;
    }
    
    /**
     * 准备导入的数据明细内容部分检查和处理
     *
     * @param Array $row
     * @param String $title
     * @param String $tmpl
     * @param Boolean $mark
     * @param Boolean $newObjFlag
     * @param String $msg
     * @return Null
     */
    function prepared_import_csv_row($row,$title,&$tmpl,&$mark,&$newObjFlag,&$msg)
    {
        $basicMaterialObj    = app::get('material')->model('basic_material');
        $salesMaterialObj    = app::get('material')->model('sales_material');
        $shopObj             = app::get('ome')->model('shop');
        $checkSalesLib    = kernel::single('material_sales_check');#数据检验有效性Lib类
        
        if (empty($row)){
            return true;
        }
        $mark = false;

        if( substr($row[0],0,1) == '*' ){
            $titleRs =  array_flip($row);
            $mark = 'title';
            #[防止重复]记录组织编码
            $this->material_bn_list      = array();
            $this->salesm_nums           = 1;
            return $titleRs;
        }else{
            $re = base_kvstore::instance('sales_material')->fetch('sales-'.$this->ioObj->cacheTime,$fileData);

            if( !$re ) $fileData = array();

            if(!$row[0]){
                $msg['error'] = "销售物料名称必须填写,销售物料编码：". $row[1];
                return false;
            }
            if(!$row[1]){
                $msg['error'] = "销售物料编码必须填写,销售物料编码：". $row[1];
                return false;
            }
            if(!$row[2]){
                $msg['error'] = "物料类型必须填写,销售物料编码：". $row[1];
                return false;
            }
            if(!$row[3]){
                $msg['error'] = "所属店铺必须填写,销售物料编码：". $row[1];
                return false;
            }

            /**
             * 赠品类型不需要填写物料售价
             * 
            if(!$row[5]){
                $msg['error'] = "物料售价必须填写,销售物料编码：". $row[1];
                return false;
            }
            **/

            if(isset($this->salesm_nums)){
                $this->salesm_nums ++;
                if($this->salesm_nums > 5000){
                    $msg['error'] = "导入的数量量过大，请减少到5000个以下！";
                    return false;
                }
            }
            
            #[防止重复]检查物料编号
            if(in_array($row[1], $this->material_bn_list))
            {
                $msg['error'] = 'Line '.$this->salesm_nums.'：物料编号【'. $row[1] .'】重复！';
                return false;
            }
            $this->material_bn_list[]    = $row[1];
            
            
            //销售物料类型(注：暂时只支持普通或赠品类型)
            if($row[2] == "普通"){
                $row[2] = 1;
            }elseif($row[2] =="促销"){
                $row[2] = 2;
            }elseif($row[2] =="赠品"){
                $row[2] = 3;
            }elseif($row[2] == "福袋"){
                $row[2] = 4;
            }elseif($row[2] == "多选一"){
                $row[2] = 5;
            }else{
                $msg['error'] = "销售物料类型填写的内容错误：".$row[2]."，请填写普通、促销、赠品、福袋、多选一";
                return false;
            }
            
            //所属店铺
            if($row[3] == '全部店铺'){
                $row[3]    = '_ALL_';
            }elseif(($row[2] == 3) && ($row[3] != '全部店铺')){
                $msg['error']    = "赠品物料类型只能选择全部店铺,销售物料编码：". $row[1];
                return false;
            }elseif($row[3]){
                $shopRow    = $shopObj->dump(array('name'=>$row[3], 's_type'=>1), 'shop_id');
                if(empty($shopRow))
                {
                    $msg['error']    = "未找到填写的所属店铺,销售物料编码：". $row[1];
                    return false;
                }
                
                $row[3]    = $shopRow['shop_id'];
            }else{
                $msg['error']    = "所属店铺不能为空,销售物料编码：". $row[1];
                return false;
            }

            #数据检查
            if(($row[2] == 2)){ //促销物料类型(至少关联一个基础物料)
                $tmp_at = array();
                //关联基础物料
                if($row[8] == ''){
                    $msg['error']    = "促销类型销售物料请至少填写一个关联的基础物料编号,销售物料编码：". $row[1];
                    return false;
                }

                $tmp_salesMInfos    = explode('|', $row[8]);
                foreach($tmp_salesMInfos as $tmp_basicMInfo){
                    $tmp_bnInfo    = explode(':', $tmp_basicMInfo);#格式：物料编号1:数量1:促销价格贡献占比|物料编号2:数量2:促销价格贡献占比

                    if (!isset($tmp_bnInfo[2])) {
                        $tmp_bnInfo[2] = 0;
                        $auto_calculation_percentage = true; #没有促销价格贡献占比参数，需要自动计算促销价格贡献占比
                    }
                    $tmp_binds[$tmp_bnInfo[0]]    = array(intval($tmp_bnInfo[1]), intval($tmp_bnInfo[2]));
                }
                unset($row[8]);

                #只有一种物料时，数量必须大于1
                foreach ($tmp_binds as $bn => $val){
                    if((count($tmp_binds) == 1) && ($val[0] < 2)){
                        $msg['error']    = "销售物料编码：". $row[1] ."行中,只有一种基础物料：". $bn ."时数量必须大于1（例格式：物料编号1:数量2）";
                        return false;
                    }
                    if ($val[0] < 1){
                        $msg['error']    = "销售物料编码：". $row[1] ."行中,基础物料：". $bn ."的数量必须大于0（例格式：物料编号1:数量1|物料编号2:数量2）";
                        return false;
                    }
                }

                //促销价格贡献占比 累加起来为100
                /*if (array_sum(array_column($tmp_binds, 1)) != 100) {
                    $msg['error']    = "销售物料编码：". $row[1] ."行中,基础物料：". $bn ."的促销价格贡献占比累加起来必须等于100（例格式：物料编号1:数量1:促销价格贡献占比1|物料编号2:数量2:促销价格贡献占比2）";
                    return false;
                }*/

                $row[8]    = $tmp_binds;
                foreach($row[8] as $bn => $val){
                    $basicInfo    = $basicMaterialObj->getList('bm_id', array('material_bn'=>$bn), 0, 1);
                    if(!$basicInfo){
                        $msg['error']    = "销售物料编码：". $row[1] ."行中找不到关联的基础物料：". $bn ."（例格式：物料编号1:数量1|物料编号2:数量2）";
                        return false;
                    }else{
                        $tmp_at[$basicInfo[0]['bm_id']]    = $val;
                    }
                }

                $sum_cost = 0;
                $material_rows = array();
                if ($auto_calculation_percentage) { #需要自动计算促销价格百分比

                    #基础物料信息
                    $material_rows = $this->getBasicMaterial(array_keys($tmp_at));
                    if (array_sum(array_column($material_rows, 'cost')) == 0) {#基础物料成本价都为0
                        $msg['error']    = "销售物料编码：". $row[1] ."行中,基础物料的成本价都为0且没有填写促销价格贡献占比,请调整基础物料成本价或者填写促销价格贡献占比并累加起来必须等于100（例格式：物料编号1:数量1:促销价格贡献占比1|物料编号2:数量2:促销价格贡献占比2）";
                        return false;
                    }

                    foreach ($tmp_at as $bm_id=> $tmp_at_item) { //计算总成本价多少
                        $cost = $material_rows[$bm_id]['cost'];
                        $number = $tmp_at_item[0];
                        $sum_cost += $cost * $number;
                    }
                }


                #促销价格贡献占比
                $row[8] = array();
                $tmp_pr = array();
                $total = 100;
                $count_basicM = count($tmp_at);
                $count_i = 1;
                foreach ($tmp_at as $key => $val){
                    if($count_basicM == 1){
                        #只有一种物料时
                        $row[8][$key]['number']    = $val[0];
                        $row[8][$key]['rate']      = $total;
                        #验证数据时使用
                        $tmp_pr[$key]    = $row[8][$key]['rate'];
                    }else{
                        #多种物料时
                        /*$row[8][$key]['number']    = $val;
                        $tmp_rate                  = intval($total / $count_basicM);
                        $row[8][$key]['rate']      = $tmp_rate;
                        if($count_basicM == $count_i){
                            $row[8][$key]['rate']      = $total - ($tmp_rate * $count_basicM) + $tmp_rate;
                        }
                        #验证数据时使用
                        $tmp_pr[$key]    = $row[8][$key]['rate'];
                        $count_i++;*/

                        if ($auto_calculation_percentage) {#自动计算促销价格百分比
                            $number = $val[0];
                            $cost = $material_rows[$key]['cost'];

                            $rate = $count_basicM == $count_i
                                ? 100 - array_sum($tmp_pr)
                                : round(($number * $cost) / $sum_cost * 100);

                            $row[8][$key]['number']  = $number;
                            $row[8][$key]['rate']    = $rate;
                            #验证数据时使用
                            $tmp_pr[$key] = $rate;
                            $count_i++;
                        } else {
                            $row[8][$key]['number']    = $val[0];
                            $row[8][$key]['rate']      = $val[1];
                            $tmp_pr[$key] = $val[1];
                        }
                    }
                }
            }elseif($row[2] == 4){ //福袋类型
                $csv_data_str = trim($row[8]);
                $tmp_lbr = array();
                //福袋基础物料组
                if(!$csv_data_str){
                    $msg['error'] = "福袋类型销售物料请至少填写一个关联的福袋组合规则,销售物料编码：". $row[1];
                    return false;
                }
                $luckybag_example_str = "，销售物料编码：".$row[1]."。参考格式（组合名称A：物料编码1|物料编码2-sku数量|送件数量|单品售价#组合名称B：物料编码1|物料编码3-sku数量|送件数量|单品售价）。";
                $tmp_luckybag_rules = explode('#', $csv_data_str);
                foreach($tmp_luckybag_rules as $var_tlr){
                    $temp_lbr = array();
                    $tmp_luckybag_name = explode(':', $var_tlr);
                    if(count($tmp_luckybag_name) == 2){
                        if(!$tmp_luckybag_name[0]){
                            $msg['error'] = "福袋组合名称异常".$luckybag_example_str; break;
                        }
                        if(!$tmp_luckybag_name[1]){
                            $msg['error'] = "福袋组合内容异常".$luckybag_example_str; break;
                        }
                        $temp_lbr["name"] = $tmp_luckybag_name[0];
                        $tmp_luckybag_detail = explode('-', $tmp_luckybag_name[1]);
                        if(count($tmp_luckybag_detail) == 2){
                            if(!$tmp_luckybag_detail[0]){
                                $msg['error'] = $temp_lbr["name"]."的福袋组合基础物料异常".$luckybag_example_str; break;
                            }
                            if(!$tmp_luckybag_detail[1]){
                                $msg['error'] = $temp_lbr["name"]."的福袋组合规则参数异常".$luckybag_example_str; break;
                            }
                            $luckybag_bm_bns = explode("|", $tmp_luckybag_detail[0]);
                            $rs_bm_ids = $basicMaterialObj->getList("bm_id",array("material_bn"=>$luckybag_bm_bns));
                            if(empty($rs_bm_ids)){
                                $msg['error'] = $temp_lbr["name"]."的福袋组合基础物料不存在".$luckybag_example_str; break;
                            }
                            if(count($rs_bm_ids) < count($luckybag_bm_bns)){
                                $msg['error'] = $temp_lbr["name"]."的福袋组合基础物料编码重复或不存在".$luckybag_example_str; break;
                            }
                            foreach($rs_bm_ids as $var_bi){
                                $temp_lbr["bm_ids"][] = $var_bi["bm_id"];
                            }
                            $luckybag_params = explode("|", $tmp_luckybag_detail[1]);
                            if(count($luckybag_params) != 3){
                                $msg['error'] = $temp_lbr["name"]."的福袋组合参数异常".$luckybag_example_str; break;
                            }
                            $temp_lbr["sku_num"] = $luckybag_params[0];
                            $temp_lbr["send_num"] = $luckybag_params[1];
                            $temp_lbr["single_price"] = $luckybag_params[2];
                        }else{
                            $msg['error'] = $temp_lbr["name"]."格式错误，".$luckybag_example_str; break;
                        }
                    }else{
                        $msg['error'] = "格式错误，".$luckybag_example_str; break;
                    }
                    $tmp_lbr[] = $temp_lbr;
                }
                if($msg['error']){
                    return false;
                }
            }elseif($row[2] == 5){ //多选一类型
                $csv_data_str = trim($row[8]);
                //福袋基础物料组
                if(!$csv_data_str){
                    $msg['error'] = "多选一类型销售物料请至少填写一个关联的基础物料规则,销售物料编码：". $row[1]; return false;
                }
                $tmp_sort = array();
                $pickone_example_str = "，销售物料编码：".$row[1]."。参考格式（随机#sku01:0|sku02:0 或 排序#sku01:10|sku02:20）。";
                $tmp_pickone_rules = explode('#', $csv_data_str);
                $count_pickone_data = count($tmp_pickone_rules);
                if($count_pickone_data <= 1 || $count_pickone_data > 2){
                    $msg['error'] = "多选一关联基础物料信息异常".$pickone_example_str; return false;
                }
                if($tmp_pickone_rules[0] == "随机"){
                    $pickone_select_type = 1;
                }elseif($tmp_pickone_rules[0] == "排序"){
                    $pickone_select_type = 2;
                }else{
                    $msg['error'] = "多选一关联基础物料选择方式信息异常".$pickone_example_str; return false;
                }
                $tmp_pickone_items = explode('|', $tmp_pickone_rules[1]);
                $arr_bm_id = array();
                foreach($tmp_pickone_items as $var_p_i){
                    $current_pickone_items = explode(':',$var_p_i);
                    if(count($current_pickone_items) != 2){
                        $msg['error'] = "多选一关联基础物料明细信息异常".$pickone_example_str; break;
                    }
                    $rs_basic = $basicMaterialObj->dump(array("material_bn"=>$current_pickone_items[0]),"bm_id");
                    if(empty($rs_basic)){
                        $msg['error'] = "基础物料编码".$current_pickone_items[0]."不存在".$pickone_example_str; break;
                    }
                    if(in_array($rs_basic["bm_id"],$arr_bm_id)){
                        $msg['error'] = "基础物料编码".$current_pickone_items[0]."重复了".$pickone_example_str; break;
                    }
                    $arr_bm_id[] = $rs_basic["bm_id"];
                    $tmp_sort[$rs_basic["bm_id"]] = $current_pickone_items[1] ? $current_pickone_items[1] : 0;
                }
                if($msg['error']){
                    return false;
                }
            }else{ #普通、赠品物料类型(默认关联的基础物料可以为空)
                if($row[8]){
                    $basicInfo    = $basicMaterialObj->getList('bm_id', array('material_bn'=>$row[8]), 0, 1);
                    if(!$basicInfo){
                        $msg['error']    = "销售物料编码：". $row[1] ."行中找不到关联的基础物料：". $row[8];
                        return false;
                    }else{
                        $row[8]    = $basicInfo[0]['bm_id'];
                    }
                }
            }
            
            #销售物料扩展信息
            $row[4]    = floatval($row[4]);
            $row[5]    = floatval($row[5]);
            $row[6]    = intval($row[6]);
            $row[7]    = trim($row[7]);
            
            #赠品物料类型_成本价和售价必须等于0
            if($row[2] == 3){
                if(($row[4] != 0) || ($row[5] != 0)){
                    $msg['error']    = "赠品物料类型成本价和售价必须等于0,销售物料编码：". $row[1];
                    return false;
                }
            }
            
            #拼接数据
            $sdf = array(
                        'sales_material_name' => $row[0],
                        'sales_material_bn' => trim($row[1]),
                        'sales_material_type' => $row[2],
                        'shop_id' => $row[3],
                        'is_bind' => 2,
                        'bind_bm_id' => $row[8],
                        'at' => $tmp_at,
                        'pr' => $tmp_pr,
                        'lbr' => $tmp_lbr,
                        'sort' => $tmp_sort,
                        'pickone_select_type' => $pickone_select_type,
                        'sales_material_bn_crc32' => '',
                        'cost' => $row[4],
                        'retail_price' => $row[5],
                        'weight' => $row[6],
                        'unit' => $row[7],
            );
            #数据有效性检查
            $err_msg = '';
            if(!$checkSalesLib->checkParams($sdf, $err_msg)){
                $msg['error']    = $err_msg .",物料编码:". $row[1];
                return false;
            }
            $fileData['salesm']['contents'][] = $sdf;
            base_kvstore::instance('sales_material')->store('sales-'.$this->ioObj->cacheTime,$fileData);
        }
        return null;
    }

    /**
     * 完成基础物料的导入
     *
     * @param Null
     * @return Null
     */
    function finish_import_csv(){
        base_kvstore::instance('sales_material')->fetch('sales-'.$this->ioObj->cacheTime,$data);
        base_kvstore::instance('sales_material')->store('sales-'.$this->ioObj->cacheTime,'');

        $oQueue = app::get('base')->model('queue');
        $aP = $data;
        $pSdf = array();

        $count = 0;
        $limit = 50;
        $page = 0;
        $orderSdfs = array();

        foreach ($aP['salesm']['contents'] as $k => $aPi){
            if($count < $limit){
                $count ++;
            }else{
                $count = 0;
                $page ++;
            }
            $pSdf[$page][] = $aPi;
        }

        foreach($pSdf as $v){
            $queueData = array(
                'queue_title'=>'销售物料导入',
                'start_time'=>time(),
                'params'=>array(
                    'sdfdata'=> $v,
                    'app' => 'material',
                    'mdl' => 'sales_material'
                ),
                'worker'=>'material_sales_material_to_import.run',
            );
            $oQueue->save($queueData);
//             $cursor_id = "111"; //测试用 直接吊队列run方法
//             kernel::single('material_sales_material_to_import')->run($cursor_id,$queueData["params"]);
        }
        $oQueue->flush();
        
        //记录日志
        $operationLogObj    = app::get('ome')->model('operation_log');
        $operationLogObj->write_log('sales_material_import@wms', 0, "批量导入销售物料,本次共导入". count($aP['salesm']['contents']) ."条记录!");
        
        return null;
    }

    /**
     * 获取基础物料信息
     * @param $bm_ids
     * @return array
     */
    private function getBasicMaterial($bm_ids)
    {
        $basic_material_ext_model = app::get('material')->model('basic_material_ext');

        $rows = $basic_material_ext_model->getList('bm_id,cost,retail_price', array('bm_id|in'=> $bm_ids), 0, -1);

        $material_rows = array();

        foreach ($rows as $row) {
            $material_rows[$row['bm_id']] = array(
                'cost'=> $row['cost'],
                'retail_price'=> $row['retail_price'],
            );
        }

        return $material_rows;
    }
}
