<?php
/**
 * 更新订单状态(所有关于订单状态的更改的公用方法)
 * @author wangyunkai
 * @version 0.1 b
 */
class ome_order {

    /**
    * 订单编辑 iframe
    * @access public
    * @param Number $order_id 订单ID
    * @param Bool $is_request 是否发起请求
    * @param Array $ext 扩展参数
    * @return Array
    */
    function update_iframe($order_id,$is_request=true,$ext=array()){
        $instance = kernel::service('service.order');
        return $instance->update_iframe($order_id,$is_request,$ext);
    }


    /**
   * 更新订单同步状态
   * @access public
   * @param number $order_id 订单ID
   * @param String $sync_status 编辑同步状态
   * @return json
   */
   function set_sync_status($order_id,$sync_status=''){
        if (!empty($order_id) && in_array($sync_status,array('fail','success'))){
           $oOrder_sync = app::get('ome')->model('order_sync_status');
           $sdf = array(
               'order_id' => $order_id,
               'sync_status' => $sync_status == 'success' ? '2' : '1',
           );
           return $oOrder_sync->save($sdf);
       }
       return false;
    }
    
    /**
    * 订单快照 （编辑前 订单存留）
    * @access public
    * @param Number $order_id 订单ID
    * @return Array
    */
    function order_photo($order_id){
        $oOrder = app::get('ome')->model('orders');
        $opObj = app::get('ome')->model('order_pmt');
        $paymentsObj = app::get('ome')->model('payments');
        $memObj = app::get('ome')->model('members');
        $data = $oOrder->dump($order_id,"*",array("order_objects"=>array("*",array("order_items"=>array("*")))));
        $data['pmt'] = $opObj->getList('*',array('order_id'=>$order_id));//订单优惠方案
        $data['payments'] = $paymentsObj->getList('*',array('order_id'=>$order_id));//支付单
        $mem_info = $memObj->getList('*',array('member_id'=>$data['member_id']));//会员信息
        $data['mem_info'] = $mem_info[0];
        return $data;
    }

    /**
    * 获取子订单的订单号
    * @access public 
    * @param String $oid 子订单号
    * @return String 订单号
    */
    function getOrderBnByoid($oid='',$node_id=''){
        if (empty($oid)) return NULL;
        
        if ($this->order_is_exists($oid,$node_id)){
            return $oid;#子订单与订单号相同,直接返回
        }

        $objModel = app::get('ome')->model('order_objects');
        $obj_detail = $objModel->getList('order_id',array('oid'=>$oid),0,1);
        if (isset($obj_detail[0]) && $order_id = $obj_detail[0]['order_id']){
            $oModel = app::get('ome')->model('orders');
            $filter = array('order_id'=>$order_id);
            if ($node_id) {
              $shopObj = kernel::single('ome_shop');
              $shops = $shopObj->getRowByNodeId($node_id);
              $shop_id = $shops['shop_id'];
              $filter['shop_id'] = $shop_id;
            }
            
            $orders = $oModel->getRow($filter,'order_bn');
            return $orders['order_bn'];
        }else{
            return NULL;
        }
    }

    /**
    * 订单号是否存在
    * @access public
    * @param String $order_bn 订单号
    * @param String $node_id 节点ID
    * @return bool
    */
    public function order_is_exists($order_bn='',$node_id=''){
        if (empty($order_bn)) return false;
        
        $filter = array('order_bn'=>$order_bn);

        $oModel = app::get('ome')->model('orders');
        if ($node_id){
            $shopObj = kernel::single('ome_shop');
            $shops = $shopObj->getRowByNodeId($node_id);
            $shop_id = $shops['shop_id'];

            $filter['shop_id'] = $shop_id;
        }
        $orders = $oModel->getRow($filter,'order_id,shop_id');
        if (isset($orders['order_id']) && $orders['order_id']){
            $shop = kernel::single('ome_shop')->getRowByShopId($orders['shop_id']);
            $orders['shop_name'] = $shop['name'];

            return $orders;
        }else{
            return false;
        }
    }

    /**
     * 发送全链路参数
     * @param Int $status 全路径状态
     * @param Int $order_id 订单序号
     * @param String $order_bn 订单号
     * @param String $remark 备注信息
     */
    public function sendMessageProduce($status, $order_id = '', $order_bn = '', $remark = '') {
        //全链路开关
        if (!(defined('MESSAGE_PRODUCE') && MESSAGE_PRODUCE)) {
            return '';
        }
        $statusList = array(
            'taobao' => array(
                0 => array('key' => 'X_TO_SYSTEM', 'value' => ''),
                1 => array('key' => 'X_SERVICE_AUDITED', 'value' => ''),
                2 => array('key' => 'X_FINANCE_AUDITED', 'value' => ''),
                3 => array('key' => 'X_ALLOCATION_NOTIFIED', 'value' => ''),
                4 => array('key' => 'X_WAIT_ALLOCATION', 'value' => ''),
                5 => array('key' => 'X_SORT_PRINTED', 'value' => ''),
                6 => array('key' => 'X_SEND_PRINTED', 'value' => ''),
                7 => array('key' => 'X_LOGISTICS_PRINTED', 'value' => ''),
                8 => array('key' => 'X_SORTED', 'value' => ''),
                9 => array('key' => 'X_EXAMINED', 'value' => ''),
                10 => array('key' => 'X_PACKAGED', 'value' => ''),
                11 => array('key' => 'X_WEIGHED', 'value' => ''),
                12 => array('key' => 'X_OUT_WAREHOUSE', 'value' => ''),
            ),
        );
        if (empty($order_id) && empty($order_bn)) {
            return false;
        }
        $ordersModel = app::get('ome')->model('orders');
        $filter = array();
        if ($order_id) {
            $filter['order_id'] = $order_id;
        }
        if ($order_bn) {
            $filter['order_bn'] = $order_bn;
        }

        $result = '';
        $orderInfo = $ordersModel->getList('order_id,order_bn,shop_id,createway', $filter);
        if ($orderInfo) {
            //线上订单
            if ($orderInfo[0]['createway'] == 'matrix') {
                //检查店铺类型是否合法
                $shopModel = app::get('ome')->model('shop');
                $shopInfo = $shopModel->getList('node_type', array('shop_id' => $orderInfo[0]['shop_id']));
                if (in_array($shopInfo[0]['node_type'], array_keys($statusList))) {
                    $tid = $orderInfo[0]['order_bn'];
                    //获得自订单
                    $orderObjectsModel = app::get('ome')->model('order_objects');
                    $orderObjectsInfo = $orderObjectsModel->getList('oid', array('order_id' => $orderInfo[0]['order_id']));
                    $orderIds = '';
                    if ($orderObjectsInfo) {
                        foreach ($orderObjectsInfo as $v) {
                            $orderIds .= strval($v['oid']) . ',';
                        }
                        $orderIds = trim(strval($orderIds), ',');
                    }
                    else {
                        $orderIds = strval($tid);
                    }
                    $statusTitle = $statusList[$shopInfo[0]['node_type']][$status]['key'];
                    $remarkTitle = empty($remark) ? $statusList[$shopInfo[0]['node_type']][$status]['value'] : $remark;
                    $dateTime = date("Y-m-d H:i:s");
                    $params = array(
                        'topic'=>'taobao_jds_TradeTrace', 
                        'tid' => $tid,
                        'order_ids' => $orderIds,
                        'status' => $statusTitle,
                        'action_time' => $dateTime,
                        'remark' => $remarkTitle,
                    );
                    $shop_id = $orderInfo[0]['shop_id'];
                    if ($shop_id) {
                        $result = kernel::single('apibusiness_router_request')->setShopId($shop_id)->add_tmc_message_produce($params);
                    }
                }
            }
        }
        return $result;
    }

    function create_order(&$sdf){
        $orderObj = app::get('ome')->model('orders');
        //判断订单号是否重复
        if($orderObj->dump(array('order_bn'=>$sdf['order_bn'],'shop_id'=>$sdf['shop_id']))){
            return false;
        }

        //开启事务，防止订单创建失败但是冻结却预占的问题
        $orderObj->db->exec('begin');

        $regionLib = kernel::single('eccommon_regions');
        //收货人/发货人地区转换
        $area = $sdf['consignee']['area'];
        $regionLib->region_validate($area);
        $sdf['consignee']['area'] = $area;
        $consigner_area = $sdf['consigner']['area'];
        $regionLib->region_validate($consigner_area);
        $sdf['consigner']['area'] = $consigner_area;

        $basicMStockLib = kernel::single('material_basic_material_stock');
        //格式化订单明细
        foreach($sdf['order_objects'] as $key=>$object){
            $object['bn'] = trim($object['bn']);
            if($object['order_items']){
                foreach($object['order_items'] as $k=>$item){
                    $item['bn'] = trim($item['bn']);
                    //货品属性
                    $product_attr = array();
                    $product_attr = $orderObj->_format_productattr($item['product_attr'], $item['product_id'],$item['original_str']);
                    $item['addon'] = $product_attr;

                    //修改预占库存
                    $num = intval($item['quantity'])-intval($item['sendnum']);
                    if($item['product_id'] > 0 && ($item['delete'] == 'false' || !isset($item['delete'])) && $num > 0){
                        $basicMStockLib->freeze($item['product_id'],$num);
                    }
                    
                    if($item['product_id'] > 0){
                        $basic_material_arr[] = $item['product_id'];
                    }

                    $object['order_items'][$k] = $item;
                }
            }
            $sdf['order_objects'][$key] = $object;
        }
        //计算实付
        $this->create_divide_pay($sdf);
        //订单可合并标记hash生成
        $combieHashIdxInfo = $this->genOrderCombieHashIdx($sdf);
        if($combieHashIdxInfo){
            $sdf['order_combine_hash'] = $combieHashIdxInfo['combine_hash'];
            $sdf['order_combine_idx'] = $combieHashIdxInfo['combine_idx'];
        }

        //判断基础物料门店是否供货，供货的标记订单为全渠道订单
        if(app::get('o2o')->is_installed()){
            if($basic_material_arr)
            {
                $basicMaterialLib    = kernel::single('material_basic_material');
                $is_omnichannel      = $basicMaterialLib->isOmnichannelOrder($basic_material_arr);
                if($is_omnichannel){
                    $sdf['omnichannel'] = 1;
                }
            }
        }else{
            unset($basic_material_arr);
        }
        
        //根据店铺取运营组织
        $shopInfo = kernel::single('ome_shop')->getRowByShopId($sdf['shop_id']);
        $sdf['org_id'] = $shopInfo['org_id'];

        if(!$orderObj->save($sdf)){
            //事务回滚
            $orderObj->db->rollBack();
            return false;
        }else{
            //事务确认
            $orderObj->db->commit();
        }

        $shopex_list = ome_shop_type::shopex_shop_type();

        if( ($sdf['source'] == 'local' || in_array($sdf['shop_type'], $shopex_list) ) && (bccomp('0.000', ($sdf['total_amount']/1),3) == 0) && $sdf['shipping']['is_cod'] != 'true'  ){ #0元订单是否需要财审.
            kernel::single('ome_order_order')->order_pay_confirm($sdf['shop_id'],$sdf['order_id'],$sdf['total_amount']);
        }

        //添加订单预占
        $basicMStockFreezeLib = kernel::single('material_basic_material_stock_freeze');
        foreach($sdf['order_objects'] as $key=>$object){
            if($object['order_items']){
                foreach($object['order_items'] as $k=>$item){
                    $num = intval($item['quantity'])-intval($item['sendnum']);
                    if($item['product_id'] > 0 && ($item['delete'] == 'false' || !isset($item['delete'])) && $num > 0){
                        $basicMStockFreezeLib->freeze($item['product_id'], material_basic_material_stock_freeze::__ORDER, 0, $sdf['order_id'], $sdf['shop_id'], 0, material_basic_material_stock_freeze::__SHARE_STORE, $num);
                    }
                }
            }
        }

        //增加订单创建日志
        $logObj = app::get('ome')->model('operation_log');
        $logObj->write_log('order_create@ome',$sdf['order_id'],'订单创建成功');
        
        //商品明细中包含福袋类型的记录日志
        foreach($sdf["order_objects"] as $var_oo){
            if($var_oo["obj_type"] == "lkb"){
                $lib_material_luckybag = kernel::single('material_luckybag');
                foreach($var_oo["order_items"] as $var_oi){
                    $arr_luckybag_log = array(
                            "order_id" => $sdf['order_id'],
                            "shop_id" => $sdf['shop_id'],
                            "lbr_id" => $var_oi['lbr_id'],
                            "sm_id" => $var_oo["product_id"] ? $var_oo["product_id"] : $var_oo['goods_id'], //售后换货生成的订单product_id 下单goods_id
                            "bm_id" => $var_oi['product_id'],
                            "num" => $var_oi['quantity'],
                            "price" => $var_oi['price'],
                            "create_time" => time(),
                    );
                    $lib_material_luckybag->deal_luckybag_log($arr_luckybag_log);
                }
            }
        }

        return true;
    }

    /**
     * 新建订单 计算实付
     * @param  array sdf order保存前的数据
     * @return void
     */
    public function create_divide_pay(&$sdf) {
        if($sdf['discount'] < 0) {
            $all_discount = $sdf['pmt_order'] - $sdf['discount'];
        } else {
            $all_discount = $sdf['pmt_order'];
        }
        $needCaculate = array();
        $saleAmount = 0;
        foreach ($sdf['order_objects'] as $k => $object) {
            if(floatval($object['divide_order_fee']) > 0){
                $needCaculate = array();
                break;
            }
            $obj_sale_price = $object['sale_price'];
            if(floatval($obj_sale_price) > 0) {
                $needCaculate[$k] = $object;
                $saleAmount += $obj_sale_price;
            }
        }
        if($needCaculate && $saleAmount) {
            $left_discount = $all_discount;
            $total_count = count($needCaculate);
            $i = 0;
            foreach ($needCaculate as $k => $object) {
                $i++;
                $obj_sale_price = $object['sale_price'];
                if($i == $total_count) {
                    $sdf['order_objects'][$k]['part_mjz_discount'] = $left_discount;
                    $sdf['order_objects'][$k]['divide_order_fee'] = bcsub($obj_sale_price, $sdf['order_objects'][$k]['part_mjz_discount'], 2);
                } else {
                    $sdf['order_objects'][$k]['part_mjz_discount'] = bcmul($all_discount, ($obj_sale_price/$saleAmount), 2);
                    $left_discount -= $sdf['order_objects'][$k]['part_mjz_discount'];
                    $sdf['order_objects'][$k]['divide_order_fee'] = bcsub($obj_sale_price, $sdf['order_objects'][$k]['part_mjz_discount'], 2);
                }
                if($object['obj_type'] == 'goods') {
                    foreach ($object['order_items'] as $ik => $item) {
                        $sdf['order_objects'][$k]['order_items'][$ik]['part_mjz_discount'] = $sdf['order_objects'][$k]['part_mjz_discount'];
                        $sdf['order_objects'][$k]['order_items'][$ik]['divide_order_fee'] = $sdf['order_objects'][$k]['divide_order_fee'];
                        break;
                    }
                }
            }
        }
    }

    /**
     * 编辑订单 计算实付
     * @param  array &$rs  编辑的明细参数
     * @param  array $post 编辑页面传输的post参数
     * @return void
     */
    public function edit_divide_pay(&$rs, $post) {
        if($post['discount'] < 0) {
            $all_pmt_order = $post['pmt_order'] - $post['discount'];
        } else {
            $all_pmt_order = $post['pmt_order'];
        }
        $send_pmt_order = 0;
        $need_divide = array();
        $need_divide_sale = 0;
        foreach ($rs['obj'] as $k => $obj) {
            if($obj['delete'] == 'true') {
                continue;
            }
            if($obj['sale_price'] > 0) {
                $need_divide_sale += $obj['sale_price'];
                $need_divide['obj-'.$k] = $obj;
            }
        }
        foreach ($rs['new'] as $k => $new) {
            if($new['sale_price'] > 0) {
                $need_divide_sale += $new['sale_price'];
                $need_divide['new-'.$k] = $new;
            }
        }
        $yu_pmt = $all_pmt_order;
        $num = count($need_divide);
        foreach ($need_divide as $n => $v) {
            list($type, $k) = explode('-', $n);
            $num--;
            if($num) {
                $part_mjz_discount = bcmul($v['sale_price']/$need_divide_sale, $all_pmt_order, 2);
                $yu_pmt -= $part_mjz_discount;
                $rs[$type][$k]['part_mjz_discount'] = $part_mjz_discount;
                $rs[$type][$k]['divide_order_fee'] = $v['sale_price'] - $part_mjz_discount;
            } else {
                $rs[$type][$k]['part_mjz_discount'] = $yu_pmt;
                $rs[$type][$k]['divide_order_fee'] = $v['sale_price'] - $yu_pmt;
            }
            if($v['obj_type'] == 'goods') {
                foreach ($v['items'] as $i => $item) {
                    $rs[$type][$k]['items'][$i]['part_mjz_discount'] = $rs[$type][$k]['part_mjz_discount'];
                    $rs[$type][$k]['items'][$i]['divide_order_fee'] = $rs[$type][$k]['divide_order_fee'];
                    break;
                }
            }
        }
    }

    /**
     * 根据传入参数重置订单分派及审核状态，主要用于发货单叫回以后的后续处理
     *
     * @param Array $param 传入参数
     * @return void
     * @author xiayuanjun
     **/
    public function resumeOrdStatus($param){

        $orderObj = app::get('ome')->model('orders');
        $combineObj = new omeauto_auto_combine();
        $dispatchObj = app::get('omeauto')->model('autodispatch');
        $dlyObj = app::get('ome')->model("delivery");

        //检查传入参数
        if(!empty($param['order_id']) && !empty($param['order_combine_idx']) && !empty($param['order_combine_hash'])){
            $orderInfo = $param;
        }elseif(!empty($param['order_id'])){
            $tmpInfo = $orderObj->getList('order_id,order_combine_idx,order_combine_hash',array('order_id'=>$param['order_id']), 0, 1);
            $orderInfo = $tmpInfo[0];
        }else{
            return;
        }

        $need_dispatch = true;
        //修改订单确认状态
        $opData['confirm'] = 'N';
        $opData['process_status'] = 'unconfirmed';
        $opData['pause'] = 'true';
        
        #[拆单]判断有部分拆分的有效发货单存在(确认状态为splitting) ExBOY
        if ($dlyObj->validDeiveryByOrderId($orderInfo['order_id']))
        {
            $opData['confirm'] = 'Y';
            $opData['process_status']   = 'splitting';
            $opData['pause']            = 'false';//因部分拆分后订单"基本信息"Tab没有操作按扭
            $need_dispatch = false;
        }
        
        $orderObj->update($opData,array('order_id'=>$orderInfo['order_id']));

        //订单状态重置后判断重新分派，放后面是因为分派的时候只认未拆分的订单，不然取规则的时候初始化订单信息会是空
        if($need_dispatch){
            $dispatchParams[] = array(
                'idx' => $orderInfo['order_combine_idx'],
                'hash' => $orderInfo['order_combine_hash'],
                'orders' => array (
                    0 => $orderInfo['order_id'],
                ),
            );
            
            //开始处理订单分派
            $result = $combineObj->dispatch($dispatchParams);
            if($result['did'] && $result['did']>0){
                $opData = $dispatchObj->dump($result['did'],'group_id,op_id');
            }else{
                $dispatchData = $dispatchObj->getList('group_id,op_id',array('defaulted'=>'true'));
                if($dispatchData && is_array($dispatchData[0])){
                    $opData = $dispatchData[0];
                }else{
                    $opData = array('group_id'=>'0','op_id'=>'0');
                }
            }

            $orderObj->update($opData,array('order_id'=>$orderInfo['order_id']));
        }
    }
    
    /**
     * 系统自动审单
     *
     * @todo 前端店铺订单进入ERP时系统自动审单
     * @param intval $order_id
     * @author wangbiao@shopex.cn
     */
    public function auto_order_combine($order_id,$sdf=array())
    {
        $order_ids      = array();
        $order_ids[]    = $order_id;
        
        if (empty($order_ids))
        {
            return false;
        }
        
        #是否开启_系统自动审单[默认:忽略可合并的订单 ]
        $cfg_combine    = app::get('ome')->getConf('ome.order.is_auto_combine');
        $cfg_merge      = app::get('ome')->getConf('ome.order.is_merge_order');
        $is_cnAuto   = app::get('ome')->getConf('ome.order.Auto');
        if($cfg_combine != 'true')
        {
            if( $sdf['cnAuto'] == 'false' ||  $is_cnAuto=='false' || empty( $sdf['cnAuto'])){
                return false;
            }
            
        }
        
        //是否选择指定时间段自动审单
        $cfg_auto_timer = app::get('ome')->getConf('ome.order.auto_timer');
        if ($cfg_auto_timer == 2){ //选择了 “指定时间” 匹配时间段
            $cfg_auto_exec_timer = app::get('ome')->getConf('ome.order.auto_exec_timer');
            $has_belong_timer = false;
            $current_time = time();
            $current_date = date('Y-m-d');
            foreach ($cfg_auto_exec_timer as $var_caet){
                $start_time_int = strtotime($current_date.' '.intval($var_caet["start_hour"]).':'.intval($var_caet["start_minute"]).':00');
                $end_time_int = strtotime($current_date.' '.intval($var_caet["end_hour"]).':'.intval($var_caet["end_minute"]).':00');
                if ($current_time >= $start_time_int && $current_time <= $end_time_int){
                    $has_belong_timer= true;
                    break;
                }
            }
            if (!$has_belong_timer){ //没有匹配到时间段
                return false;
            }
        }
        
        //获取system账号信息
        $opinfo = kernel::single('ome_func')->get_system();
        
        //自动审单_批量日志
        $blObj  = app::get('ome')->model('batch_log');
        
        $batch_number = count($order_ids);
        $bldata = array(
                'op_id' => $opinfo['op_id'],
                'op_name' => $opinfo['op_name'],
                'createtime' => time(),
                'batch_number' => $batch_number,
                'log_type'=> 'combine',
                'log_text'=> serialize($order_ids)
        );
        $result = $blObj->save($bldata);
        
        //自动审单_任务队列(改成多队列多进程)
        if (defined('SAAS_COMBINE_MQ') && SAAS_COMBINE_MQ == 'true') {
            $data = array();
            $data['spider_data']['url'] = kernel::openapi_url('openapi.autotask','service');

            $push_params = array(
                'log_text'  => $bldata['log_text'],
                'log_id'    => $bldata['log_id'],
                'task_type' => 'autorder',
            );
            $push_params['sign'] =  taskmgr_rpc_sign::gen_sign($push_params);
            foreach ($push_params as $key => $val) {
                $postAttr[] = $key . '=' . urlencode($val);
            }

            $data['spider_data']['params']    = empty($postAttr) ? '' : join('&', $postAttr);
            $data['relation']['to_node_id']   = base_shopnode::node_id('ome');
            $data['relation']['from_node_id'] = '0';
            $data['relation']['tid']          = $bldata['log_id'];
            $data['relation']['to_url']       = $data['spider_data']['url'];
            $data['relation']['time']         = time();

            $routerKey = 'tg.order.combine.'.$data['nodeId'];

            $message = json_encode($data);
            $mq = kernel::single('base_queue_mq');
            $mq->connect($GLOBALS['_MQ_COMBINE_CONFIG'], 'TG_COMBINE_EXCHANGE', 'TG_COMBINE_QUEUE');
            $mq->publish($message, $routerKey);
            $mq->disConnect();
        } else {
            $push_params = array(
                'data' => array(
                        'log_text'  => $bldata['log_text'],
                        'log_id'    => $bldata['log_id'],
                        'task_type' => 'autorder',
                ),
                'url' => kernel::openapi_url('openapi.autotask','service')
            );
            
            kernel::single('taskmgr_interface_connecter')->push($push_params);  
        }

        return true;
    }

    /**
     * 货到付款订单签收后自动支付
     *
     * @param int $deliveryId 发货单ID
     * @author wangbiao@shopex.cn
     */
    public function codAutoPay($deliveryId) {
        $objOrder = app::get('ome')->model('orders');
        $oPayment = app::get('ome')->model('payments');
        $orderDelivery = app::get('ome')->model('delivery_order')->getList('*',array('delivery_id'=>$deliveryId));
        $arrOrderId = array();
        foreach($orderDelivery as $od) {
            $arrOrderId[] = $od['order_id'];
        }
        $filter = array('order_id'=>$arrOrderId,'is_cod'=>'true','pay_status'=>'0');
        $arrOrderData = $objOrder->getList('order_id,total_amount,ship_status,shop_id,currency,source', $filter);
        $arrShopId = array();
        foreach($arrOrderData as $val) {
            $arrShopId[$val['shop_id']] = $val['shop_id'];
        }
        $shop = app::get('ome')->model('shop')->getList('shop_id, node_id, node_type', array('shop_id'=>$arrShopId));
        $arrShopData = array();
        foreach($shop as $val) {
            $arrShopData[$val['shop_id']] = $val;
        }
        $objsales = app::get('ome')->model('sales');
        $saleData = $objsales->getList('sale_id,order_id', array('order_id' => $arrOrderId));
        $orderSale = array();
        foreach($saleData as $sale) {
            $orderSale[$sale['order_id']][] = $sale['sale_id'];
        }
        $paymentCfg =  app::get('ome')->model('payment_cfg')->getList('id', array('pay_bn'=>'offline'), 0, 1);
        $c2c_shop = ome_shop_type::shop_list();
        foreach ($arrOrderData as $order) {
            $shop_detail = $arrShopData[$order['shop_id']];
            if ($shop_detail['node_id'] && !in_array($shop_detail['node_type'],$c2c_shop) && $order['source'] == 'matrix'){
                $params = array (
                    'order_id' => $order['order_id'],
                    'shop_id' => $order['shop_id'],
                    'money' => $order['total_amount'],
                    'pay_type' => 'offline',
                    'payment' => intval($paymentCfg[0]['id']),
                    'memo' => '签收COD订单系统自动确认支付',
                );
                $oPayment->payment_request($params);
                continue;
            }
            $orderData = array();
            $orderData['payed']      = $order['total_amount'];
            $orderData['pay_status'] = 1;
            $orderData['paytime']    = time();
            $orderData['payment']    = '线下支付';
            if($order['ship_status'] == 1) {
                $orderData['status'] = 'finish';
                if($orderSale[$order['order_id']]) {
                    $objsales->update(array('paytime' => $orderData['paytime']), array('order_id' => $order['order_id']));
                }
            }
            $orderFilter = array('order_id'=>$order['order_id']);
            $objOrder->update($orderData, $orderFilter);
            //日志
            $memo = '签收COD订单自动确认支付';
            app::get('ome')->model('operation_log')->write_log('order_modify@ome',$order['order_id'],$memo);
            //生成支付单
            $payment_bn = $oPayment->gen_id();
            $paymentdata = array();
            $paymentdata['payment_bn'] = $payment_bn;
            $paymentdata['order_id'] = $order['order_id'];
            $paymentdata['shop_id'] = $order['shop_id'];
            $paymentdata['currency'] = $order['currency'];
            $paymentdata['money'] = $orderData['payed'];
            $paymentdata['paycost'] = 0;
            $curr_time = time();
            $paymentdata['t_begin'] = $curr_time;//支付开始时间
            $paymentdata['t_end'] = $curr_time;//支付结束时间
            $paymentdata['cur_money'] = $orderData['payed'];
            $paymentdata['pay_type'] = 'offline';
            $paymentdata['ip'] = kernel::single("base_request")->get_remote_addr();
            $paymentdata['status'] = 'succ';
            $paymentdata['memo'] = '签收COD订单系统自动确认支付';
            $paymentdata['is_orderupdate'] = 'false';
            $oPayment->create_payments($paymentdata);
            //日志
            app::get('ome')->model('operation_log')->write_log('payment_create@ome',$paymentdata['payment_id'],'生成支付单');
        }
    }

    public function genOrderCombieHashIdx($params){

        $order_combine_hash = '';
        $order_combine_idx = '';
        $member_id = $params['member_id'];
        $order_bn = $params['order_bn'];
        $shop_id = $params['shop_id'];
        $ship_name = $params['consignee']['name'];
        $ship_mobile = $params['consignee']['mobile'];
        $ship_area = $params['consignee']['area'];
        $ship_addr = $params['consignee']['addr'];
        $order_source = $params['order_source'];
        $ship_tel = $params['consignee']['telephone'];
        $is_cod = $params['shipping']['is_cod'];
        $self_delivery = $params['self_delivery'];
        $shop_type = $params['shop_type'];

        $uniqueness = $member_id ? $member_id : $order_bn;
        $uniqueness2 = $order_source == 'tbdx' ? $order_id : $order_source;
        $uniqueness3 = $is_cod == 'true' ? $order_id : $is_cod;
        $uniqueness4 = $self_delivery == 'false' ? $order_id : $self_delivery;

        switch($shop_type){
            case 'taobao':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$uniqueness2.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx= $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                break;
            case 'dangdang':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$uniqueness3.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx= $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                break;
            case 'amazon':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$uniqueness4.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                break;
            case 'aikucun':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$order_id.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                break;
            case 'shopex_b2b':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
                break;
            case '360buy':
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$uniqueness3.'-'.$ship_tel.'-'.$shop_type;
                $combine_idx = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod.'-'.$ship_tel.'-'.$shop_type;
            default:
                $combine_hash = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod;
                $combine_idx = $uniqueness.'-'.$shop_id.'-'.$ship_name.'-'.$ship_mobile.'-'.$ship_area.'-'.$ship_addr.'-'.$is_cod;
                break;
        }

        $result['combine_hash'] = MD5($combine_hash);
        $result['combine_idx'] = CRC32($combine_idx);
        return $result;
    }
    
    //格式化新建订单items层数据
    public function format_order_items_data($item_type,$obj_number,$basicMInfos){
        $weight = 0;
        $order_items = array();
        foreach($basicMInfos as $basicMInfo){
            $current_order_items = array(
                "product_id" => $basicMInfo['bm_id'],
                "bn" => $basicMInfo['material_bn'],
                "name" => $basicMInfo['material_name'],
                "sendnum" => 0,
                "item_type" => $item_type,
            );
            switch($item_type){
                case "pkg": //促销
                    $current_order_items["price"] = $basicMInfo['rate_price'] ? bcdiv($basicMInfo['rate_price'], $basicMInfo['number']*$obj_number, 2) : 0.00;
                    $current_order_items["amount"] = $basicMInfo['rate_price'] ? $basicMInfo['rate_price'] : 0.00;
                    $current_order_items["sale_price"] = $basicMInfo['rate_price'] ? $basicMInfo['rate_price'] : 0.00;
                    $current_order_items["quantity"] = $basicMInfo['number']*$obj_number;
                    $weight += $basicMInfo['weight']*$basicMInfo['number']*$obj_number;
                    break;
                case "lkb": //福袋
                    $current_order_items["price"] = $basicMInfo['price'];
                    $current_order_items["amount"] = $basicMInfo['price']*$obj_number*$basicMInfo['number'];
                    $current_order_items["sale_price"] = $basicMInfo['price']*$obj_number*$basicMInfo['number'];
                    $current_order_items["quantity"] = $basicMInfo['number']*$obj_number;
                    $current_order_items["lbr_id"] = $basicMInfo["lbr_id"];
                    $weight += $basicMInfo['weight']*$basicMInfo['number']*$obj_number;
                    break;
                case "pko": //多选一
                    $current_order_items["price"] = $basicMInfo['price'];
                    $current_order_items["amount"] = $basicMInfo['price']*$basicMInfo['number'];
                    $current_order_items["sale_price"] = $basicMInfo['sale_price']*$basicMInfo['number'];
                    $current_order_items["quantity"] = $basicMInfo['number'];
                    $weight += $basicMInfo['weight']*$basicMInfo['number'];
                    break;
                default: //赠品 普通销售物料
                    $current_order_items["price"] = $basicMInfo['price'];
                    $current_order_items["amount"] = $basicMInfo['price']*$obj_number;
                    $current_order_items["sale_price"] = $basicMInfo['sale_price']*$obj_number;
                    $current_order_items["quantity"] = $basicMInfo['number']*$obj_number;
                    $weight += $basicMInfo['weight']*$basicMInfo['number']*$obj_number;
                    break;
            }
            $order_items[] = $current_order_items; //组order_items
        }
        return array("order_items" => $order_items,"weight" => $weight);
    }
    
    /**
     * 根据已拆分的发货单货品数量,判断订单的拆分状态
     *
     * @param int $order_id
     * @param string
     */
    function get_order_process_status($order_id)
    {
        $deliveryObj = app::get('ome')->model('delivery');
        
        //默认为"未确认"状态
        $process_status = 'unconfirmed';
        
        //获取已生成的发货单delivery_id
        $delivery_ids = $deliveryObj->getDeliverIdByOrderId($order_id);
        if($delivery_ids)
        {
            //订单明细货品总数量
            $sql = "SELECT sum(nums) AS nums FROM sdb_ome_order_items WHERE order_id=". $order_id ." AND `delete`='false'";
            $tempData = $deliveryObj->db->selectrow($sql);
            $orderItemNums = $tempData['nums'];
            
            //发货单货品总数量
            $sql = "SELECT sum(number) AS nums FROM sdb_ome_delivery_items WHERE delivery_id IN(". implode(',', $delivery_ids) .")";
            $tempData = $deliveryObj->db->selectrow($sql);
            $dlyItemNums = $tempData['nums'];
            
            if($orderItemNums == $dlyItemNums){
                $process_status = 'splited';
            }else{
                $process_status = 'splitting';
            }
        }
        
        return $process_status;
    }
}
