<?php
/**
* 换货类
*
* @author chenping<chenping@shopex.cn>
*/
class ome_reship{

    function __construct($app){
        $this->app = $app;
    }

    /**
     * 取消退款申请
     * @param String $apply_id 退款申请ID
     * @param String $memo 取消理由
     * @return void
     * @author
     **/
    public function cancelRefundApply($apply_id,$memo=''){
        $refundApplyModel = $this->app->model('refund_apply');
        $applyUpdate = array(
            'status' => '3',
            'memo'=>$memo
        );
        $refundApplyModel->update($applyUpdate,array('apply_id'=>$apply_id));
    }

    /**
     * @description 退换货申请退款生成退款单据
     * @access public
     */
    public function createRefund($refundApply,$order){
        # 更新退款金额
        $orderModel = $this->app->model('orders');
        $payed = $order['payed'] - $refundApply['money'];
        $payed = ( $payed > 0 ) ? $payed : 0;
        $orderModel->update(array('payed'=>$payed),array('order_id'=>$order['order_id']));

        $opLogModel = $this->app->model('operation_log');
        $opLogModel->write_log('order_modify@ome',$order['order_id'],"售后退款成功，更新订单退款金额。系统自动操作，退款金额用于支付新订单。");

        # 退款申请单处理
        $refundApplyUpdate = array(
            'status' => '4',
            'refunded' => $refundApply['money'],
            'last_modified' => time(),
            'account' => $refundApply['account'],
            'pay_account' => $refundApply['pay_account'],
        );
        $refundApplyModel = $this->app->model('refund_apply');
        $refundApplyModel->update($refundApplyUpdate,array('apply_id'=>$refundApply['apply_id']));

        $opLogModel->write_log('refund_apply@ome',$refundApply['apply_id'],"售后退款成功，更新退款申请状态。系统自动操作，退款金额用于支付新订单。");

        # 退款单处理
        $paymethods = ome_payment_type::pay_type();
        $opInfo = kernel::single('ome_func')->getDesktopUser();
        $refunddata = array(
            'refund_bn' => $refundApply['refund_apply_bn'],
            'order_id' => $order['order_id'],
            'shop_id' => $order['shop_id'],
            'account' => $refundApply['account'],
            'bank' => $refundApply['bank'],
            'pay_account' => $refundApply['pay_account'],
            'currency' => $order['currency'],
            'money' => $refundApply['money'],
            'paycost' => 0,
            'cur_money' => $refundApply['money'],
            'pay_type' => $refundApply['pay_type'],
            'payment' => $refundApply['payment'],
            'paymethod' => $paymethods[$refundApply['pay_type']],
            'op_id' => $opInfo['op_id'],
            't_ready' => time(),
            't_sent' => time(),
            'memo' => $refundApply['memo'],
            'status' => 'succ',
            'refund_refer' => '1',
            'return_id' => $refundApply['return_id'],
        );
        if ($refundApply['archive'] && $refundApply['archive']=='1') {
            $refunddata['archive'] = '1';
        }
        $oRefund = $this->app->model('refunds');
        $oRefund->save($refunddata);

        // 更新订单支付状态
        kernel::single('ome_order_func')->update_order_pay_status($order['order_id']);
        $opLogModel->write_log('refund_accept@ome',$refunddata['refund_id'],"售后退款成功，生成退款单".$refunddata['refund_bn']."，退款金额用于支付新订单。");
    }

    /**
     * 取消补差价订单
     * @param Int $order_id 订单ID
     * @param String $shop_id 店铺ID
     **/
    public function cancelDiffOrder($order_id,$shop_id,$memo=''){
        define('FRST_TRIGGER_OBJECT_TYPE','订单：订单作为补差价订单取消');
        define('FRST_TRIGGER_ACTION_TYPE','ome_ctl_admin_order：do_cancel');

        $c2c_shop_list = ome_shop_type::shop_list();

        $node_type = $this->app->model('shop')
                        ->select()->columns('node_type')
                        ->where('shop_id=?',$shop_id)
                        ->instance()->fetch_one();

        $mod = in_array($node_type,$c2c_shop_list) ? 'async' : 'sync';

        return $this->app->model('orders')->cancel($order_id,$memo,true,$mod);
    }

    /**
     * 对换货订单进行支付操作
     * @param Array $order 订单信息
     **/
    public function payChangeOrder($order){
        $mathLib      = kernel::single('eccommon_math');
        $orderModel   = $this->app->model('orders');
        $paymentModel = $this->app->model('payments');

        $orderdata = array(
            'order_id' => $order['order_id'],
            'pay_status' => $order['pay_status'],
            'paytime' => time(),
        );

        # 支付配置
        //$cfg = $this->app->model('payment_cfg')->dump();
        $cfg = array();

        $orderdata['pay_bn'] = $cfg['pay_bn'];

        $orderdata['payed'] = $mathLib->getOperationNumber($order['pay_money']);

        $orderdata['payment'] = '线下支付';

        $orderModel->update($orderdata,array('order_id'=>$order['order_id']));

        //日志
        $memo = '做质检时连带操作;订单付款操作,用订单('.$order['reship_order_bn'].')的退款金额作支付金额';
        $oOperation_log = $this->app->model('operation_log');
        $oOperation_log->write_log('order_modify@ome',$order['order_id'],$memo);

        //生成支付单
        $payment_bn = $paymentModel->gen_id();
        $paymentdata = array();
        $paymentdata['payment_bn']  = $payment_bn;
        $paymentdata['order_id']    = $order['order_id'];
        $paymentdata['shop_id']     = $order['shop_id'];
        $paymentdata['account']     = '';
        $paymentdata['bank']        = '';
        $paymentdata['pay_account'] = '';
        $paymentdata['currency']    = $order['currency'];
        $paymentdata['money']       = $order['pay_money'];
        $paymentdata['paycost']     = 0;
        $curr_time                  = time();
        $paymentdata['t_begin']     = $curr_time;//支付开始时间
        $paymentdata['t_end']       = $curr_time;//支付结束时间
        $paymentdata['trade_no']    = '';//支付网关的内部交易单号，默认为空
        $paymentdata['cur_money']   = $paymentdata['money'];
        $paymentdata['pay_type']    = 'offline';
        $paymentdata['payment']     = '';
        $paymentdata['paymethod']   = '线下支付';
        $paymentdata['payment_refer'] = '1';

        $opInfo = kernel::single('ome_func')->getDesktopUser();
        $paymentdata['op_id'] = $opInfo['op_id'];
        if ($order['archive'] && $order['archive'] == '1') {
            //$paymentdata['archive'] = '1';
        }
        $paymentdata['ip'] = kernel::single("base_request")->get_remote_addr();
        $paymentdata['status'] = 'succ';
        $paymentdata['memo'] = '做质检时连带操作;系统生成换货订单支付单据;通过退款金额进行支付;补换货的订单:'.$order['reship_order_bn'];
        $paymentdata['is_orderupdate'] = 'false';
        $paymentModel->create_payments($paymentdata);

        //日志
        $oOperation_log->write_log('payment_create@ome',$paymentdata['payment_id'],'生成支付单');
    }

    /**
     * @description 判断是否为反审单据
     * @access public
     */
    public function is_precheck_reship($is_check,$need_sv='true'){
        return ($is_check=='0' && $need_sv == 'false') ? true : false;
    }

    /**
     * @description 更新订单的状态,并把发货状态同步到前端
     **/
    function updatediffOrder($order_bn){
        $mdl_ome_orders = $this->app->model('orders');
        $mdl_ome_delivery_order = app::get('ome')->model('delivery_order');
        $order_detail = $mdl_ome_orders->dump(array('order_bn'=>$order_bn),'order_id,shop_id,order_bn,ship_name,ship_time,ship_mobile,ship_zip,ship_area,ship_tel,ship_email,ship_addr');
        $rs_ome_delivery_order = $mdl_ome_delivery_order->dump(array('order_id'=>$order_detail["order_id"]));
        if(!empty($rs_ome_delivery_order)){ //有发货单直接返回
            return;
        }
        //补差价订单生成发货单 不生成出入库明细和销售单
        $this->deliver_by_diff_order($order_detail);
    }
    
    //补差价订单生成发货单 不生成出入库明细和销售单
    function deliver_by_diff_order($diff_order_info){
        $mdl_ome_orders = $this->app->model('orders');
        $mdl_ome_reship = app::get('ome')->model('reship');
        $rs_reship = $mdl_ome_reship->dump(array("diff_order_bn"=>$diff_order_info["order_bn"]));
        //直接添加已发货的发货单 组参数
        $mdl_ome_delivery = app::get('ome')->model("delivery");
        $mdl_order_objects = app::get('ome')->model('order_objects');
        $mdl_order_items = app::get('ome')->model('order_items');
        $mdl_operation_log = app::get('ome')->model('operation_log');
        $delivery = array('consignee' => $diff_order_info["consignee"]);
        $itemFilter = array('order_id'=>$diff_order_info["order_id"], 'delete'=>'false');
        $object_list = $mdl_order_objects->getList('*',$itemFilter);
        $delivery_params = array();
        foreach($object_list as $object){
            $itemFilter['obj_id'] = $object['obj_id'];
            $orderItems = $mdl_order_items->getList('*', $itemFilter);
            foreach($orderItems as $item){
                if($item['product_id'] > 0){
                    if($item['nums'] > 0){
                        $delivery_params['branch_id'] = $rs_reship['branch_id'];
                        $delivery_params['logi_no'] = $rs_reship['logi_no'];
                        $delivery_params['logi_name'] = $rs_reship['logi_name'];
                        $delivery_params['items'][] = array(
                            'product_id' => $item['product_id'],
                            'shop_product_id' => $item['shop_product_id'],
                            'bn' => $item['bn'],
                            'number' => $item['nums'],
                            'product_name' => $item['name'],
                            'spec_info' => $item['addon'],
                        );
                        $delivery_params['order_items'][] = array(
                            'product_id' => $item['product_id'],
                            'bn' => $item['bn'],
                            'number' => $item['nums'],
                            'product_name' => $item['name'],
                            'item_id' => $item['item_id'],
                        );
                    }
                }else{
                    $mdl_operation_log->write_log('order_confirm@ome',$diff_order_info["order_id"],'明细未修复，无法生成发货单');
                    return false;
                }
            }
        }
        $delivery["branch_id"] = $delivery_params["branch_id"];
        $delivery["delivery_waybillCode"] = $diff_order_info["order_bn"]; //logi_no给order_bn
        $delivery['delivery_items'] = $delivery_params['items'];
        $order_items = $delivery_params['order_items'];
        $split_status = '';
        $delivery_id = $mdl_ome_delivery->addDelivery($diff_order_info["order_id"],$delivery,array(),$order_items,$split_status,true);
        if ($delivery_id) {
            //更新补差价订单状态
            $update_arr = array(
                'ship_status' => '1',
                'status' => 'finish',
                'archive' => '1',
                'process_status' => 'confirmed',
                'confirm' => 'Y',
                'is_delivery' => 'Y',
                'logi_no' => $diff_order_info["order_bn"], //logi_no给order_bn
            );
            $mdl_ome_orders->update($update_arr,array('order_bn'=>$diff_order_info["order_bn"]));
            //标记已发货数
            $sql = 'UPDATE `sdb_ome_order_items` SET sendnum=nums WHERE `delete`="false" AND order_id='.$diff_order_info['order_id'];
            $mdl_ome_orders->db->exec($sql);
            $mdl_operation_log->write_log('order_confirm@ome',$diff_order_info['order_id'],"售后补差价订单，自动生成发货单成功。");
            //发货状态回传 参照class ome_event_trigger_shop_delivery的public function delivery_confirm_retry($orderids)
            $deliveryList = $mdl_ome_delivery->getList('*',array('delivery_id'=>$delivery_id));
            if(!$deliveryList){
                return;
            }
            $delivery = $deliveryList[0];
            $mdl_ome_delivery_order = app::get('ome')->model('delivery_order');
            $deliveryOrderList = $mdl_ome_delivery_order->getList('delivery_id,order_id',array('delivery_id'=>$delivery_id));
            $order_ids = array(); $delivery_orders = array();
            foreach ($deliveryOrderList as $key => $value) {
                $order_ids[] = $value['order_id'];
                $delivery_orders[$value['delivery_id']] = &$orderList[$value['order_id']];
            }
            $rows = $mdl_ome_orders->getList('*',array('order_id'=>$order_ids));
            foreach($rows as $key => $value) {
                $orderList[$value['order_id']] = $value;
            }
            $sdf = kernel::single('ome_event_trigger_shop_data_delivery_router')
                    ->set_shop_id($delivery['shop_id'])
                    ->init($deliveryList,$delivery_orders)
                    ->get_sdf($delivery['delivery_id']);
            if (!$sdf){
                return;
            }
            kernel::single('erpapi_router_request')->set('shop',$delivery['shop_id'])->delivery_add($sdf);
            kernel::single('erpapi_router_request')->set('shop',$delivery['shop_id'])->delivery_logistics_update($sdf);
            kernel::single('erpapi_router_request')->set('shop',$delivery['shop_id'])->delivery_confirm($sdf);
        }else{
            $mdl_operation_log->write_log('order_confirm@ome',$diff_order_info['order_id'],"生成发货单失败");
        }
    }
    
    //残损确认判断 （来自save_check方法）目前只支持电商仓
    public function check_defective($reship_id,$is_check,&$msg){
        //9 => '拒绝质检'  10 => '质检异常' 11 => '待确认'
        if($is_check == '10' || $is_check == '11' || $is_check == '9'){
            $oReship_item = app::get('ome')->model('reship_items');
            $normal_reship_item = $oReship_item->getList('*',array('reship_id'=>$reship_id,'normal_num|than'=>0),0,1);
            $reship_item = $oReship_item->getList('*',array('reship_id'=>$reship_id,'defective_num|than'=>0),0,1);
            if (count($normal_reship_item)==0 && count($reship_item)==0) {
                $msg = '良品或不良品数量至少有一种不为0!';
                return false;
            }
            if (count($reship_item)>0) {
                $damaged = kernel::single('console_iostockdata')->getDamagedbranch($reship_item[0]['branch_id']);
                if (!$damaged) {
                    $msg = '由于有不良品入库，请设置主仓对应残仓';
                    return false;
                }
            }
        }
        return true;
    }
    
    //质检成功后 拒绝质检 质检异常 待确认 状态的 执行相应的操作  目前只支持电商仓
    public function part_finish_aftersale($reship_id,$is_check,&$msg){
        //9 => '拒绝质检'  10 => '质检异常' 11 => '待确认'
        if($is_check == '10' || $is_check == '11' || $is_check == '9'){
            $Oreship = app::get('ome')->model('reship');
            if($Oreship->finish_aftersale($reship_id)){
                $result = kernel::single('console_reship')->siso_iostockReship($reship_id);
                if($is_check == '10'){ //质检异常
                    //反审核质检
                    $process_sql = "UPDATE sdb_ome_return_process_items SET is_check='true' WHERE reship_id=".$reship_id." AND is_check='false'";
                    $Oreship->db->exec($process_sql);
                }
                if(!$result){
                    $msg = '没有生成出入库明细!';
                    return false;
                }
            }
        }
        return true;
    }
    
    /**
     * 审核退换货单
     *
     * @param array $params 退换货单相关参数
     * @param string $error_msg
     *
     * @todo exec_type 审核类型(1:系统自动审核,2:批量审核),默认为0
     * @return bool
     */
    function confirm_reship($params, &$error_msg)
    {
        $Oreship = app::get('ome')->model('reship');
        $oReship_item = app::get('ome')->model('reship_items');
        $oOperation_log = app::get('ome')->model('operation_log');
        $orderObj = app::get('ome')->model('orders');
        
        $consoleReshipLib = kernel::single('console_reship');
        $reReshipLib = kernel::single('ome_receipt_reship');
        $libBranchProduct = kernel::single('ome_branch_product');
        
        $operation = '审核退换货单,';
        $reship_id = $params['reship_id'];
        $is_auto_check = false;
        
        //check
        if(empty($reship_id) || !isset($params['status'])){
            $error_msg = '无效的操作';
            return false;
        }
        
        
        //审核类型(1:系统自动审核,2:批量审核),默认值0为人工操作
        $execTypeList = array(1=>'系统自动', 2=>'批量');
        $exec_type = $params['exec_type'] ? intval($params['exec_type']) : 0;
        if($exec_type){
            $operation = $execTypeList[$exec_type] . $operation;
            
            $is_auto_check = true;
        }
        
        //审核状态
        $status = $params['status'];
        
        //是否反审核
        $is_anti = ($params['is_anti'] == true) ? true : false;
        
        
        //filter
        $filter = array('reship_id'=>$reship_id);
        
        //退换货单信息
        $reship = $Oreship->dump($filter, 'order_id, is_check, reship_bn, return_type, reason, need_sv, source, changebranch_id, branch_id, totalmoney, tmoney, return_id,archive,change_status');
        if(empty($reship)){
            $error_msg = '没有可操作的退换货单!';
            return false;
        }
        
        $order_id = $reship['order_id'];
        
        //是否归档
        $is_archive = ($reship['archive']==1 ? true : false);
        $source = $reship['source']; //归档来源(archive、matrix)
        
        //[归档]订单实例化
        if($is_archive){
            $orderObj = app::get('archive')->model('orders');
        }
        
        //check
        if(!$is_anti && $reship['is_check'] == '1'){
            $error_msg = '退换货单是已审核状态,不能操作!'; //反审核时,不用判断is_check=1状态
            return false;
        }
        
        if ($reship['is_check']=='7') {
            $error_msg = '退换货单是已完成状态,不能操作!';
            return false;
        }
        
        //全额退款订单判断
        $current_order_arr = $orderObj->dump(array('order_id'=>$reship['order_id']), 'pay_status');
        if($current_order_arr['pay_status'] == '5'){
            if($reship['tmoney'] > 0){
                $error_msg = '此订单已经全额退款，退款金额不能大于已支付金额!';
                return false;
            }
            if($reship['return_type'] == 'change'){
                $error_msg = '此订单已经全额退款，不能进行换货操作!';
                return false;
            }
        }
        
        //判断订单是否是门店仓履约的
        //退入仓为判断依据 因为只有是退入仓是门店仓的情况下 换出仓才可能是门店仓 走wap端  当退入仓为电商仓时，无论换出仓是门店还是电商仓都走电商oms端
        $store_id = kernel::single('ome_branch')->isStoreBranch($reship["branch_id"]);
        
        
        //换货需验证退入和换出的仓库
        if($reship["return_type"] == "change"){
            $result_branch_check = kernel::single('o2o_return')->check_reship_branch($reship["branch_id"],$reship["changebranch_id"],$error_msg);
            if(!$result_branch_check){
                $error_msg = '退换货单：'. $reship['reship_bn'] .','. $error_msg;
                return false;
            }
        }
        
        //自动审核时检查金额
        if($is_auto_check){
            
            /***
            //如果"最后合计金额"为0元则重算
            if($reship['return_type'] == 'return'){
                if(empty($reship['totalmoney']) || (bccomp('0.000', $reship['totalmoney'], 3) == 0)){
                    if($reship['return_id']){
                        kernel::single('ome_return_rchange')->update_totalmoney($reship['return_id']);
                        
                        //重新读取数据
                        $reship = $Oreship->dump($filter, 'order_id, is_check, reship_bn, return_type, reason, need_sv, source, changebranch_id, totalmoney, tmoney, return_id');
                    }
                }
            }
            ***/
            
            //判断退款金额
            $orderInfo = $orderObj->getList('order_bn, total_amount, payed', array('order_id'=>$order_id), 0, 1);
            $orderInfo = $orderInfo[0];
            if ($reship['archive'] == '1'){
                $orderInfo = app::get('archive')->model('orders')->dump(array('order_id'=>$order_id),'order_bn, total_amount, payed');
            }
            if(empty($orderInfo)){
                $error_msg = '退换货单：'. $reship['reship_bn'] .',关联订单不存在!';
                return false;
            }
            
            if($reship['totalmoney'] > $orderInfo['payed']){
                $error_msg = '退款金额不能大于订单的已支付金额!';
                return false;
            }
            
            if ($reship['return_type'] == 'return' && $reship['totalmoney'] < 0) {
                $error_msg = '退款金额不能小于零!';
                return false;
            }
        }
        
        //检查退换货单明细
        if($status == '1'){
            //判断退换货单明细
            $is_fail = $this->check_reship_items($reship_id);
            if($is_fail){
                $error_msg = '退货明细有货号不存在';
                return false;
            }
            
            //[换货]检查退入商品和换出商品的数量
            if($reship['return_type'] == 'change'){
                $result = array();
                $oReship_item->Get_items_count($reship_id, $result);
                
                if($result['return'] == '0' || ($result['change'] == '0' && $reship['change_status']!='2')){
                    $error_msg = '退换货单：'. $reship['reship_bn'] .',退入商品与换货商品数量有误!';
                    return false;
                }
                if($reship['change_status']!='2'){
                    if($result['return'] != $result['change']){
                        $error_msg = '退换货单：'. $reship['reship_bn'] .',退入商品数量与换货商品数量不一致!';
                        return false;
                    }
                }
            }
        }
        
        
        //[换货]判断库存
        if ((($reship['is_check']=='0' && $reship['need_sv'] == 'true') || $reship['is_check']=='12') && ($reship['return_type'] == 'change')){
            $change_item = $consoleReshipLib->change_items($reship_id);
            $changebranch_id = $reship['changebranch_id'];
            
            $change_store_id = kernel::single('ome_branch')->isStoreBranch($changebranch_id);
            
            foreach ($change_item as $item) {
                
                //是否检查库存数 默认true(门店仓存在不管控库存的情况)
                $check_stock = true;
                
                if($change_store_id)
                {
                    //门店仓换货库存检查
                    $arr_stock = kernel::single('o2o_return')->o2o_store_stock($changebranch_id, $item['product_id']);
                    $could_usable_store = $arr_stock["store"]; //值可能会包括 "-" "x" 或 真实的库存数
                    
                    if ($could_usable_store == "x"){
                        $error_msg = '单据号：'. $reship['reship_bn'] .'; 货号:'.$item['bn'].',与此门店仓无供货关系';
                        return false;
                    }elseif($could_usable_store == "-"){
                        $check_stock = false; //不管控库存
                    }
                }
                else
                {
                    //电商仓换货库存检查
                    $usable_store = $libBranchProduct->getAvailableStore($changebranch_id, array($item['product_id']));
                    
                    $could_usable_store = $usable_store[$item['product_id']] ? $usable_store[$item['product_id']] : 0;
                }
                
                if ($item['num'] > $could_usable_store && $check_stock) {
                    $error_msg = '单据号：'. $reship['reship_bn'] .'; 货号:'.$item['bn'].',可用库存不足';
                    return false;
                }
            }
        }
        
        //[电商仓]最终确认时,残损确认判断
        if (!$store_id){
            $result = $this->check_defective($reship_id, $reship['is_check'], $check_error_msg);
            if(!$result){
                $error_msg = '单据号：'. $reship['reship_bn'] .';'. $check_error_msg;
                return false;
            }
        }
        
        //判断重复审核(1分钟之内不能重复)
        if($status == '1' && in_array($reship['is_check'], array('0', '2'))){
            $cacheKeyName = sprintf("confirm_reship_id_%s", $reship_id);
            $cacheData = cachecore::fetch($cacheKeyName);
            if ($cacheData === false) {
                cachecore::store($cacheKeyName, $reship_id, 60);
            }else{
                $error_msg = '退换货单：'. $reship['reship_bn'] .',已经在审核中,一分钟内请不要重复操作!';
                return false;
            }
        }
        
        //update data
        $updateData = array('is_check'=>$status);
        
        //post
        if($_POST){
            //审核原因
            $reason = unserialize($reship['reason']);
            if(isset($_POST['reason'])) {
                $reason['check'] = $_POST['reason'];
            }
            
            if(isset($_POST['need_sv'])) {
                $updateData['need_sv'] = $_POST['need_sv'];
            }
            
            $updateData['reason'] = serialize($reason);
        }
        
        if ($reship['is_check']=='0' && $reship['need_sv'] == 'true') {
            $updateData['check_time'] = time(); //审核时间
        }
        
        $affect_row = $Oreship->update($updateData, array('reship_id'=>$reship_id));
        if(!is_numeric($affect_row) || $affect_row <= 0){
            $error_msg = '退换货单：'. $reship['reship_bn'] .' 审核失败!';
            return false;
        }
        
        //log
        $schema = $Oreship->schema['columns'];
        $memo = $operation. '状态:'.$schema['is_check']['type'][$status];
        $oOperation_log->write_log('reship@ome', $reship_id, $memo);
        
        
        //库存管控 换货预占电商仓or门店仓的库存
        if (($reship['is_check']=='0' && $reship['need_sv'] == 'true') || $reship['is_check']=='12') {
            if ($reship['return_type'] == 'change' && $reship['changebranch_id']) {
                $storeManageLib = kernel::single('ome_store_manage');
                $storeManageLib->loadBranch(array('branch_id'=>$reship['changebranch_id']));
                $params_stock = array(
                        "params" => array_merge($reship,array('reship_id'=>$reship_id)),
                        "node_type" => "checkChangeReship",
                );
                $storeManageLib->processBranchStore($params_stock, $err_msg);
            }
            
            //发起通知单
            ome_return_notice::create($reship_id);
        }
        
        
        //[电商仓]质检成功后执行相应的操作
        if (!$store_id){
            $result = $this->part_finish_aftersale($reship_id,$reship['is_check'], $after_error_msg);
            if(!$result){
                $error_msg = '退换货单：'. $reship['reship_bn'] . $after_error_msg;
                return false;
            }
        }
        
        return true;
    }

    /**
     * 执行队列任务自动审核退货单
     *
     * @param int $reship_id 售后单ID
     *
     * @return bool
     */
    function batch_reship_queue($reship_id)
    {
        $reship_ids = array();
        $reship_ids[0] = $reship_id;

        //获取system账号信息
        $opinfo = kernel::single('ome_func')->get_system();

        //自动审单_批量日志
        $blObj  = app::get('ome')->model('batch_log');

        $batch_number = count($reship_ids);
        $bldata = array(
                'op_id' => $opinfo['op_id'],
                'op_name' => $opinfo['op_name'],
                'createtime' => time(),
                'batch_number' => $batch_number,
                'log_type'=> 'confirm_reship',
                'log_text'=> serialize($reship_ids),
        );
        $result = $blObj->save($bldata);

        //自动审批任务队列(改成多队列多进程)
        if (defined('SAAS_COMBINE_MQ') && SAAS_COMBINE_MQ == 'true') {
            $data = array();
            $data['spider_data']['url'] = kernel::openapi_url('openapi.autotask','service');

            $push_params = array(
                    'log_text'  => $bldata['log_text'],
                    'log_id'    => $bldata['log_id'],
                    'task_type' => 'confirmreship',
            );
            $push_params['taskmgr_sign'] = taskmgr_rpc_sign::gen_sign($push_params);
            foreach ($push_params as $key => $val) {
                $postAttr[] = $key . '=' . urlencode($val);
            }

            $data['spider_data']['params']    = empty($postAttr) ? '' : join('&', $postAttr);
            $data['relation']['to_node_id']   = base_shopnode::node_id('ome');
            $data['relation']['from_node_id'] = '0';
            $data['relation']['tid']          = $bldata['log_id'];
            $data['relation']['to_url']       = $data['spider_data']['url'];
            $data['relation']['time']         = time();

            $routerKey = 'tg.order.reship.'. $data['relation']['from_node_id'];

            $message = json_encode($data);
            $mq = kernel::single('base_queue_mq');
            $mq->connect($GLOBALS['_MQ_COMBINE_CONFIG'], 'TG_COMBINE_EXCHANGE', 'TG_COMBINE_QUEUE');
            $mq->publish($message, $routerKey);
            $mq->disConnect();
        } else {
            $push_params = array(
                    'data' => array(
                            'log_text'  => $bldata['log_text'],
                            'log_id'    => $bldata['log_id'],
                            'task_type' => 'confirmreship',
                    ),
                    'url' => kernel::openapi_url('openapi.autotask','service'),
            );

            kernel::single('taskmgr_interface_connecter')->push($push_params);
        }

        return true;
    }

    static function check_reship_items($reship_id){
        $db = kernel::database();

        $sql = "SELECT reship_id FROM sdb_ome_reship_items WHERE reship_id=".$reship_id." AND (product_id IS NULL OR product_id=0)";


        $items = $db->selectrow($sql);
        return $items;
    }
    
    /**
     * 退换货单上更新了退回物流单号时,同步给第三方仓储
     */
    public function request_wms_returnorder($reship_id, &$error_msg){
        $oOperation_log = $this->app->model('operation_log');
        $Oreship = app::get('ome')->model('reship');
        
        $reshipInfo = $Oreship->dump(array('reship_id'=>$reship_id), 'reship_id, is_check, reship_bn');
        if (in_array($reshipInfo['is_check'], array('4', '5', '7'))) {
            $error_msg = '无需同步给第三方仓储';
            return false;
        }
        
        $reship_data = kernel::single('ome_receipt_reship')->reship_create(array('reship_id'=>$reship_id));
        
        $wms_id = kernel::single('ome_branch')->getWmsIdById($reship_data['branch_id']);
        
        $rsp_result = kernel::single('console_event_trigger_reship')->create($wms_id, $reship_data, false);
        
        //记录日志
        $rsp_result = json_encode($rsp_result);
        $memo = '发送至第三方:';
        $oOperation_log->write_log('reship@ome', $reship_id, $memo.$rsp_result);
        
        return true;
    }
}