<?php
/**
 * @description 电商仓库存处理类
 * @access public
 * @author xiayuanjun@shopex.cn
 * @ver 0.1
 */
class ome_store_manage_branch extends ome_store_manage_abstract implements ome_store_manage_interface{

    function __construct($is_ctrl_store)
    {
        parent::__construct($is_ctrl_store);

        $this->_libBranchProduct = kernel::single('ome_branch_product');
        $this->_basicMaterialStock = kernel::single('material_basic_material_stock');
        $this->_basicMStockFreezeLib = kernel::single('material_basic_material_stock_freeze');
    }

    /**
     * 添加发货单节点的库存处理方法
     *
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     * @return boolean true/false
     */
    public function addDly($params, &$err_msg){
      
        $branchObj        = app::get('ome')->model("branch");
        $basicMStockFreezeLib  = kernel::single('material_basic_material_stock_freeze');

        $delivBranch  = $branchObj->getDelivBranch($params['branch_id']);
        $branchIds    = $delivBranch[$params['branch_id']]['bind_conf'];
        $branchIds[]  = $params['branch_id'];

        //增加branch_product的冻结库存
        foreach($params['delivery_items'] as $key=>$items)
        {
            $sql = "SELECT product_id, branch_id, store FROM sdb_ome_branch_product
                    WHERE product_id=".$items['product_id']." AND branch_id IN (".implode(',', $branchIds).")";
            $branch_p = $branchObj->db->select($sql);

            //可用库存
            $store_num    = 0;
            if($branch_p)
            {
                foreach ((array) $branch_p as $row) {
                    //根据仓库ID、基础物料ID获取该物料仓库级的预占
                    $store_freeze    = $basicMStockFreezeLib->getBranchFreeze($row['product_id'], $row['branch_id']);
                    $row['store']    = ($row['store'] < $store_freeze) ? 0 : ($row['store'] - $store_freeze);
                    $store_num    += $row['store'];
                }
            }

            if (!is_numeric($items['number']))
            {
                $err_msg    .= $items['product_name'] .":请输入正确数量";
                return false;
            }
            if ($params['order_type']!='platform'){
                if (empty($store_num) || $store_num == 0 ||  $store_num < $items['number'])
                {
                    $err_msg    .= $items['product_name'] .":商品库存不足";
                    return false;
                }

            }
            

            //订单货品预占释放
            $rs = $this->_basicMStockFreezeLib->unfreeze($items['product_id'], material_basic_material_stock_freeze::__ORDER, 0, $params['order_id'], '', material_basic_material_stock_freeze::__SHARE_STORE, $items['number']);
            if ($rs == false){
                $err_msg    .= '订单货品冻结释放失败!';
                return false;
            }

            //添加仓库预占流水
            $rs = $this->_basicMStockFreezeLib->freeze($items['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__DELIVERY, $params['delivery_id'], $params['shop_id'], $params['branch_id'], material_basic_material_stock_freeze::__SHARE_STORE, $items['number']);

            if ($rs == false) {
                $err_msg    .= '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /**
     * 取消发货单节点的库存处理方法
     *
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     * @return boolean true/false
     */
    public function cancelDly($params, &$err_msg){

        foreach($params['delivery_items'] as $key=>$dly_item)
        {
            //释放仓库预占流水
            $rs = $this->_basicMStockFreezeLib->unfreeze($dly_item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__DELIVERY, $params['delivery_id'], $params['branch_id'], material_basic_material_stock_freeze::__SHARE_STORE, $dly_item['number']);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }

            //添加订单货品预占流水
            $rs = $this->_basicMStockFreezeLib->freeze($dly_item['product_id'], material_basic_material_stock_freeze::__ORDER, 0, $params['order_id'], $params['shop_id'], 0, material_basic_material_stock_freeze::__SHARE_STORE, $dly_item['number']);
            if ($rs == false) {
                $err_msg    = '订单货品冻结预占失败!';
                return false;
            }
        }

        //发货单取消，电商仓预占流水删除
        $this->_basicMStockFreezeLib->delDeliveryFreeze($params['delivery_id']);

        return true;
    }

    /**
     * 发货单发货节点的库存处理方法
     *
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     * @return boolean true/false
     */
    public function consignDly($params, &$err_msg){

        $delivery_items = $params['delivery_items'];
        $nitems = array();
        foreach($delivery_items as $item){
            if (isset($nitems[$item['product_id']])){
                $nitems[$item['product_id']]['number']+=$item['number'];
            }else{
                $nitems[$item['product_id']]= array(
                    'product_id'    =>  $item['product_id'],
                    'number'        =>  $item['number'],
                );
            }
            

        }

        ksort($nitems);

        foreach($nitems as $key=>$dly_item)
        {
            $rs    = $this->_basicMaterialStock->unfreeze($dly_item['product_id'], $dly_item['number']);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            //释放仓库预占流水
            $rs = $this->_basicMStockFreezeLib->unfreeze($dly_item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__DELIVERY, $params['delivery_id'], $params['branch_id'], material_basic_material_stock_freeze::__SHARE_STORE, $dly_item['number']);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //发货完成，电商仓预占流水删除
        $this->_basicMStockFreezeLib->delDeliveryFreeze($params['delivery_id']);

        //电商仓发货，仓库货品实际库存扣减跟着出入库明细生成，所以不在这里处理

        return true;
    }

    /**
     * 订单暂停发货单取消节点的库存处理方法
     *
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     * @return boolean true/false
     */
    public function pauseOrd($params, &$err_msg){

        foreach($params['delivery_items'] as $key=>$dly_item)
        {
            //释放仓库预占流水
            $rs = $this->_basicMStockFreezeLib->unfreeze($dly_item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__DELIVERY, $params['delivery_id'], $params['branch_id'], material_basic_material_stock_freeze::__SHARE_STORE, $dly_item['number']);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }

            //添加订单货品预占流水
            $rs = $this->_basicMStockFreezeLib->freeze($dly_item['product_id'], material_basic_material_stock_freeze::__ORDER, 0, $params['order_id'], $params['shop_id'], 0, material_basic_material_stock_freeze::__SHARE_STORE, $dly_item['number']);
            if ($rs == false) {
                $err_msg    = '订单货品冻结预占失败!';
                return false;
            }
        }

        //发货完成，电商仓预占流水删除
        $this->_basicMStockFreezeLib->delDeliveryFreeze($params['delivery_id']);

        return true;
    }

    /**
     * 订单恢复节点的库存处理方法
     *
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     * @return boolean true/false
     */
    public function renewOrd($params, &$err_msg){
        //订单暂停、第三方仓发货单直接走的是撤销，如果订单恢复后不需要相应的货品库存处理逻辑
    }

    /*
     * 售后：退换货单审核库存处理方法
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function checkChangeReship($params, &$err_msg){

        $reship_id  = $params['reship_id'];
        $branch_id  = $params['changebranch_id'];
        $log_type   = 'reship';

        if(empty($reship_id) || empty($branch_id))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        $reship_item = kernel::single('console_reship')->change_items($reship_id);
        if(empty($reship_item))
        {
            $err_msg    = '没有换货明细!';
            return false;
        }

        //增加预占
        foreach($reship_item as $item )
        {
            $product_id = $item['product_id'];
            $num        = $item['num'];

            $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '+', $log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RESHIP, $reship_id, '', $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /*
     * 售后：拒绝换货单库存处理方法（质检拒绝质检/wap换货单确认拒绝）
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function refuseChangeReship($params, &$err_msg){

        $reship_id  = $params['reship_id'];
        $branch_id  = $params['changebranch_id'];
        $log_type   = 'reship';

        if(empty($reship_id) || empty($branch_id))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        $reship_item = kernel::single('console_reship')->change_items($reship_id);
        if(empty($reship_item))
        {
            $err_msg    = '没有换货明细!';
            return false;
        }

        //释放库存预占
        foreach($reship_item as $item )
        {
            $product_id = $item['product_id'];
            $num        = $item['num'];

            $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', $log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RESHIP, $reship_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($reship_id, material_basic_material_stock_freeze::__RESHIP);

        return true;
    }

    /*
     * 售后：确认退换货单库存处理方法（质检确认收货的退入/wap退换货单确认的退入）
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function confirmReshipReturn($params, &$err_msg){
        //实际库存增加
        return kernel::single('ome_return_process')->do_iostock($params['por_id'],1,$err_msg);
    }

    /*
     * 售后：确认换货单库存处理方法（质检确认收货的换出/wap换货单确认的换出）
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function confirmReshipChange($params, &$err_msg){

        $reship_id  = $params['reship_id'];
        $branch_id  = $params['changebranch_id'];
        $log_type   = 'reship';

        if(empty($reship_id) || empty($branch_id))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        $reship_item = kernel::single('console_reship')->change_items($reship_id);
        if(empty($reship_item))
        {
            $err_msg    = '没有换货明细!';
            return false;
        }

        //释放库存预占
        foreach($reship_item as $item )
        {
            $product_id = $item['product_id'];
            $num        = $item['num'];

            $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', $log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RESHIP, $reship_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($reship_id, material_basic_material_stock_freeze::__RESHIP);

        return true;
    }

    /*
     * 退换货单回传拒绝换货库存处理方法
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function reshipReturnRefuseChange($params, &$err_msg){

        $reship_id  = $params['reship_id'];
        $branch_id  = $params['changebranch_id'];
        $log_type   = 'reship';

        if(empty($reship_id) || empty($branch_id))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        $reship_item = kernel::single('console_reship')->change_items($reship_id);
        if(empty($reship_item))
        {
            $err_msg    = '没有换货明细!';
            return false;
        }

        //释放库存预占
        foreach($reship_item as $item )
        {
            $product_id = $item['product_id'];
            $num        = $item['num'];

            $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', $log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RESHIP, $reship_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($reship_id, material_basic_material_stock_freeze::__RESHIP);

        return true;
    }

    /*
     * 最终收货确认由换货变为退货库存处理方法
     * @param array $params 传入参数
     * @param string $err_msg 错误信息
     */
    public function editChangeToReturn($params, &$err_msg){

        $reship_id  = $params['reship_id'];
        $branch_id  = $params['changebranch_id'];
        $log_type   = 'reship';

        if(empty($reship_id) || empty($branch_id))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        $reship_item = kernel::single('console_reship')->change_items($reship_id);
        if(empty($reship_item))
        {
            $err_msg    = '没有换货明细!';
            return false;
        }

        //释放库存预占
        foreach($reship_item as $item )
        {
            $product_id = $item['product_id'];
            $num        = $item['num'];

            $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', $log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RESHIP, $reship_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($reship_id, material_basic_material_stock_freeze::__RESHIP);

        return true;
    }

    /**
     * 审核采购退货
     */
    public function checkReturned($params, &$err_msg)
    {
        $rp_id      = $params['rp_id'];
        $branch_id  = $params['branch_id'];
        $log_type   = 'return_purchase';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['num'], '+', 'return_purchase');
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RETURNED, $rp_id, '', $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['num'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /**
     * 完成采购退货
     */
    public function finishReturned($params, &$err_msg)
    {
        $rp_id      = $params['rp_id'];
        $branch_id  = $params['branch_id'];
        $log_type   = 'return_purchase';
        $product_id = $params['product_id'];
        $num        = $params['num'];

        $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', 'return_purchase');
        if ($rs == false)
        {
            $err_msg    = '货品冻结释放失败!';
            return false;
        }

        $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RETURNED, $rp_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
        if ($rs == false) {
            $err_msg    = '仓库货品冻结释放失败!';
            return false;
        }

        return true;
    }

    /**
     * 取消采购退货
     */
    public function cancelReturned($params, &$err_msg)
    {
        $rp_id      = $params['rp_id'];
        $branch_id  = $params['branch_id'];
        $log_type   = 'return_purchase';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['num'], '-');
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__RETURNED, $rp_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['num'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($rp_id, material_basic_material_stock_freeze::__RETURNED);

        return true;
    }

    /**
     * 审核调拨出库单
     */
    public function checkStockout($params, &$err_msg)
    {
        $iso_id     = $params['iso_id'];
        $branch_id  = $params['branch_id'];
        $log_type   = 'other';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['nums'], '+', 'other');
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__STOCKOUT, $iso_id, '', $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['nums'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /**
     * 最终处理调拨出库单
     */
    public function finishStockout($params, &$err_msg)
    {
        $iso_id     = $params['iso_id'];
        $branch_id  = $params['branch_id'];
        $log_type   = 'other';
        $product_id = $params['product_id'];
        $num        = $params['num'];

        $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', 'other');
        if ($rs == false)
        {
            $err_msg    = '货品冻结释放失败!';
            return false;
        }

        $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__STOCKOUT, $iso_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
        if ($rs == false) {
            $err_msg    = '仓库货品冻结释放失败!';
            return false;
        }

        return true;
    }

    /**
     * 审核库内转储单
     */
    public function saveStockdump($params, &$err_msg)
    {
        $stockdump_id = $params['stockdump_id'];
        $branch_id    = $params['branch_id'];
        $log_type     = 'stockdump';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['num'], '+', 'stockdump');
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__STOCKDUMP, $stockdump_id, '', $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['num'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /**
     * 最终处理库内转储单
     */
    public function finishStockdump($params, &$err_msg)
    {
        $stockdump_id = $params['stockdump_id'];
        $branch_id    = $params['branch_id'];
        $log_type     = 'stockdump';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['num'], '-', 'stockdump');
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__STOCKDUMP, $stockdump_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['num'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结释放失败!';
                return false;
            }
        }

        //删除预占流水
        $this->_basicMStockFreezeLib->delOtherFreeze($stockdump_id, material_basic_material_stock_freeze::__STOCKDUMP);

        return true;
    }

    /**
     * 审核唯品会出库单
     */
    public function checkVopstockout($params, &$err_msg)
    {
        $stockout_id = $params['stockout_id'];
        $branch_id   = $params['branch_id'];
        $log_type    = 'vop';

        if(empty($branch_id) || empty($params['items']))
        {
            $err_msg    = '无效操作!';
            return false;
        }

        foreach($params['items'] as $key => $item)
        {
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($item['product_id'], $item['num'], '+', 'vop');
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($item['product_id'], material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__VOPSTOCKOUT, $stockout_id, '', $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $item['num'], $log_type);
            if ($rs == false) {
                $err_msg    = '仓库货品冻结预占失败!';
                return false;
            }
        }

        return true;
    }

    /**
     * 最终处理唯品会出库单
     */
    public function finishVopstockout($params, &$err_msg)
    {
        $stockout_id = $params['stockout_id'];
        $branch_id   = $params['branch_id'];
        $log_type    = 'vop';
        $product_id = $params['product_id'];
        $num        = $params['num'];

        $rs = $this->_basicMaterialStock->chg_product_store_freeze($product_id, $num, '-', 'vop');
        if ($rs == false)
        {
            $err_msg    = '货品冻结释放失败!';
            return false;
        }

        $rs = $this->_basicMStockFreezeLib->unfreeze($product_id, material_basic_material_stock_freeze::__BRANCH, material_basic_material_stock_freeze::__VOPSTOCKOUT, $stockout_id, $branch_id, material_basic_material_stock_freeze::__SHARE_STORE, $num, $log_type);
        if ($rs == false) {
            $err_msg    = '仓库货品冻结释放失败!';
            return false;
        }

        return true;
    }
    
    //人工库存预占
    public function artificialFreeze($params, &$err_msg){
        $log_type = "artificialfreeze";
        foreach ($params as $var_p){
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($var_p['bm_id'],$var_p['freeze_num'],'+',$log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结预占失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->freeze($var_p["bm_id"],material_basic_material_stock_freeze::__BRANCH,material_basic_material_stock_freeze::__ARTIFICIALFREEZE,$var_p["obj_id"],"",$var_p["branch_id"],material_basic_material_stock_freeze::__SHARE_STORE,$var_p["freeze_num"],$log_type);
            if ($rs == false) {
                $err_msg = '仓库货品冻结预占失败!';
                return false;
            }
        }
        return true;
    }
    
    //人工库存预占释放
    public function artificialUnfreeze($params, &$err_msg){
        $log_type = "artificialunfreeze";
        $obj_ids = array();
        foreach ($params as $var_p){
            $rs = $this->_basicMaterialStock->chg_product_store_freeze($var_p['bm_id'],$var_p['freeze_num'],'-',$log_type);
            if ($rs == false)
            {
                $err_msg    = '货品冻结释放失败!';
                return false;
            }

            $rs = $this->_basicMStockFreezeLib->unfreeze($var_p["bm_id"],material_basic_material_stock_freeze::__BRANCH,material_basic_material_stock_freeze::__ARTIFICIALFREEZE,$var_p["bmsaf_id"],$var_p["branch_id"],material_basic_material_stock_freeze::__SHARE_STORE,$var_p["freeze_num"],$log_type);
            if ($rs == false) {
                $err_msg = '仓库货品冻结释放失败!';
                return false;
            }
            $obj_ids[] = $var_p["bmsaf_id"];
        }
        //删除预占流水
        $this->_basicMStockFreezeLib->delArtificialFreeze($obj_ids);
        return true;
    }
    
}
